@@ -17,7 +17,7 @@
 *  Content:
 *       This example demonstrates use of oneAPI Math Kernel Library (oneMKL)
 *       API oneapi::mkl::dft to perform 2-D Single Precision Real to Complex
-*       Fast-Fourier Transform on a SYCL device (Host, CPU, GPU).
+*       Fast-Fourier Transform on a SYCL device (CPU, GPU).
 *
 *       The supported floating point data types for data are:
 *           float
@@ -76,7 +76,7 @@ static void init_r(float *data, int N1, int N2, int H1, int H2)
 }
 
 // Verify that data has unit peak at H
-static int verify_c(float* data,
+static int verify_c(const float* data,
                     int N1, int N2, int H1, int H2)
 {
     // Note: this simple error bound doesn't take into account error of
@@ -137,7 +137,7 @@ static void init_c(float *data, int N1, int N2, int H1, int H2)
 }
 
 /* Verify that data has unit peak at H */
-static int verify_r(float* data,
+static int verify_r(const float* data,
                     int N1, int N2, int H1, int H2)
 {
     // Generalized strides for row-major addressing of data
@@ -174,7 +174,7 @@ static int verify_r(float* data,
     return SUCCESS;
 }
 
-static int verify_round(float *data, int N1, int N2, int H1, int H2)
+static int verify_round(const float *data, int N1, int N2, int H1, int H2)
 {
     // Generalized strides for row-major addressing of data
     int S1 = 1, S2 = N1;
@@ -213,7 +213,7 @@ static int verify_round(float *data, int N1, int N2, int H1, int H2)
     return SUCCESS;
 }
 
-int run_dft_forward_example(cl::sycl::device &dev) {
+int run_dft_forward_example(sycl::device &dev) {
     //
     // Initialize data for DFT
     //
@@ -235,11 +235,11 @@ int run_dft_forward_example(cl::sycl::device &dev) {
     //
     try {
         // Catch asynchronous exceptions
-        auto exception_handler = [] (cl::sycl::exception_list exceptions) {
+        auto exception_handler = [] (sycl::exception_list exceptions) {
             for (std::exception_ptr const& e : exceptions) {
                 try {
                     std::rethrow_exception(e);
-                } catch(cl::sycl::exception const& e) {
+                } catch(sycl::exception const& e) {
                     std::cout << "Caught asynchronous SYCL exception:" << std::endl
                               << e.what() << std::endl;
                 }
@@ -247,11 +247,11 @@ int run_dft_forward_example(cl::sycl::device &dev) {
         };
 
         // create execution queue with asynchronous error handling
-        cl::sycl::queue queue(dev, exception_handler);
+        sycl::queue queue(dev, exception_handler);
 
         // Setting up SYCL buffer and initialization
-        cl::sycl::buffer<float, 1> xBuffer(x, cl::sycl::range<1>(N1*N2));
-        cl::sycl::buffer<float, 1> yBuffer(cl::sycl::range<1>((N1/2 + 1)*N2*2));
+        sycl::buffer<float, 1> xBuffer(x, sycl::range<1>(N1*N2));
+        sycl::buffer<float, 1> yBuffer(sycl::range<1>((N1/2 + 1)*N2*2));
         xBuffer.set_write_back(false);
         yBuffer.set_write_back(false);
 
@@ -270,7 +270,7 @@ int run_dft_forward_example(cl::sycl::device &dev) {
         std::cout<<"\tUsing SYCL buffers"<<std::endl;
         oneapi::mkl::dft::compute_forward(desc, xBuffer, yBuffer);
 
-        auto yAcc = yBuffer.get_access<cl::sycl::access::mode::read>();
+        auto yAcc = yBuffer.get_host_access(sycl::read_only);
         buffer_result = verify_c(yAcc.get_pointer(), N1, N2, H1, H2);
 
         // Using USM
@@ -285,7 +285,7 @@ int run_dft_forward_example(cl::sycl::device &dev) {
         free(x_usm, queue.get_context());
         free(y_usm, queue.get_context());
     }
-    catch(cl::sycl::exception const& e) {
+    catch(sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during FFT" << std::endl;
         std::cout << "\t\t" << e.what() << std::endl;
         std::cout << "\t\tError code: " << get_error_code(e) << std::endl;
@@ -299,7 +299,7 @@ int run_dft_forward_example(cl::sycl::device &dev) {
     return result;
 }
 
-int run_dft_backward_example(cl::sycl::device &dev) {
+int run_dft_backward_example(sycl::device &dev) {
     //
     // Initialize data for DFT
     //
@@ -321,11 +321,11 @@ int run_dft_backward_example(cl::sycl::device &dev) {
     //
     try {
         // Catch asynchronous exceptions
-        auto exception_handler = [] (cl::sycl::exception_list exceptions) {
+        auto exception_handler = [] (sycl::exception_list exceptions) {
             for (std::exception_ptr const& e : exceptions) {
                 try {
                     std::rethrow_exception(e);
-                } catch(cl::sycl::exception const& e) {
+                } catch(sycl::exception const& e) {
                     std::cout << "Caught asynchronous SYCL exception:" << std::endl
                               << e.what() << std::endl;
                 }
@@ -333,11 +333,11 @@ int run_dft_backward_example(cl::sycl::device &dev) {
         };
 
         // create execution queue with asynchronous error handling
-        cl::sycl::queue queue(dev, exception_handler);
+        sycl::queue queue(dev, exception_handler);
 
         // Setting up SYCL buffer and initialization
-        cl::sycl::buffer<float, 1> xBuffer(x, cl::sycl::range<1>((N1/2 + 1)*N2*2));
-        cl::sycl::buffer<float, 1> yBuffer(cl::sycl::range<1>(N1*N2));
+        sycl::buffer<float, 1> xBuffer(x, sycl::range<1>((N1/2 + 1)*N2*2));
+        sycl::buffer<float, 1> yBuffer(sycl::range<1>(N1*N2));
         xBuffer.set_write_back(false);
         yBuffer.set_write_back(false);
 
@@ -355,7 +355,7 @@ int run_dft_backward_example(cl::sycl::device &dev) {
         // Using SYCL buffers
         std::cout<<"\tusing SYCL buffers"<<std::endl;
         oneapi::mkl::dft::compute_backward(desc, xBuffer, yBuffer);
-        auto yAcc = yBuffer.get_access<cl::sycl::access::mode::read>();
+        auto yAcc = yBuffer.get_host_access(sycl::read_only);
         buffer_result = verify_r(yAcc.get_pointer(), N1, N2, H1, H2);
 
         // Using USM
@@ -370,7 +370,7 @@ int run_dft_backward_example(cl::sycl::device &dev) {
         free(x_usm, queue.get_context());
         free(y_usm, queue.get_context());
     }
-    catch(cl::sycl::exception const& e) {
+    catch(sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during FFT" << std::endl;
         std::cout << "\t\t" << e.what() << std::endl;
         std::cout << "\t\tError code: " << get_error_code(e) << std::endl;
@@ -384,7 +384,7 @@ int run_dft_backward_example(cl::sycl::device &dev) {
     return result;
 }
 
-int run_dft_roundtrip_example(cl::sycl::device &dev) {
+int run_dft_roundtrip_example(sycl::device &dev) {
     //
     // Initialize data for DFT
     //
@@ -407,22 +407,22 @@ int run_dft_roundtrip_example(cl::sycl::device &dev) {
     //
     try {
         // Catch asynchronous exceptions
-        auto exception_handler = [] (cl::sycl::exception_list exceptions) {
+        auto exception_handler = [] (sycl::exception_list exceptions) {
             for (std::exception_ptr const& e : exceptions) {
                 try {
                     std::rethrow_exception(e);
-                } catch(cl::sycl::exception const& e) {
+                } catch(sycl::exception const& e) {
                     std::cout << "Caught asynchronous SYCL exception:" << std::endl
                               << e.what() << std::endl;
                 }
             }
         };
-        cl::sycl::queue queue(dev, exception_handler);
+        sycl::queue queue(dev, exception_handler);
         // Setting up buffer and initialization
 
-        cl::sycl::buffer<float, 1> realInBuffer(x, sycl::range<1>(N2*N1));
-        cl::sycl::buffer<float, 1> complexOutBuffer(sycl::range<1>(N2*(N1/2 + 1)*2));
-        cl::sycl::buffer<float, 1> realOutBuffer(sycl::range<1>(N2*N1));
+        sycl::buffer<float, 1> realInBuffer(x, sycl::range<1>(N2*N1));
+        sycl::buffer<float, 1> complexOutBuffer(sycl::range<1>(N2*(N1/2 + 1)*2));
+        sycl::buffer<float, 1> realOutBuffer(sycl::range<1>(N2*N1));
         realInBuffer.set_write_back(false);
         complexOutBuffer.set_write_back(false);
         realOutBuffer.set_write_back(false);
@@ -462,7 +462,7 @@ int run_dft_roundtrip_example(cl::sycl::device &dev) {
         oneapi::mkl::dft::compute_backward(desc, complexOutBuffer, realOutBuffer);
         
         std::cout<<"\tusing SYCL buffers"<<std::endl;
-        buffer_result = verify_round((realOutBuffer.get_access<cl::sycl::access::mode::read>()).
+        buffer_result = verify_round((realOutBuffer.get_host_access(sycl::read_only)).
                                                 get_pointer(), N1, N2, H1, H2);
 
         // Using USM
@@ -479,7 +479,7 @@ int run_dft_roundtrip_example(cl::sycl::device &dev) {
         free(complexOutUSM, queue.get_context());
         free(realOutUSM, queue.get_context());
     }
-    catch(cl::sycl::exception const& e) {
+    catch(sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during FFT" << std::endl;
         std::cout << "\t\t" << e.what() << std::endl;
         std::cout << "\t\tError code: " << get_error_code(e) << std::endl;
@@ -516,10 +516,9 @@ void print_example_banner() {
 // Main entry point for example.
 //
 // Dispatches to appropriate device types as set at build time with flag:
-// -DSYCL_DEVICES_host -- only runs host implementation
 // -DSYCL_DEVICES_cpu -- only runs SYCL CPU implementation
 // -DSYCL_DEVICES_gpu -- only runs SYCL GPU implementation
-// -DSYCL_DEVICES_all (default) -- runs on all: host, cpu and gpu devices
+// -DSYCL_DEVICES_all (default) -- runs on all: cpu and gpu devices
 //
 //  For each device selected and each supported data type, Basic_Sp_R2C_2D_FFTExample
 //  runs is with all supported data types
@@ -532,7 +531,7 @@ int main() {
 
     int returnCode = 0;
     for (auto it = list_of_devices.begin(); it != list_of_devices.end(); ++it) {
-        cl::sycl::device my_dev;
+        sycl::device my_dev;
         bool my_dev_is_found = false;
         get_sycl_device(my_dev, my_dev_is_found, *it);
 
@@ -572,5 +571,6 @@ int main() {
         }
     }
 
+    mkl_free_buffers();
     return returnCode;
 }
