@@ -17,7 +17,7 @@
 *  Content:
 *       This example demonstrates use of oneAPI Math Kernel Library (oneMKL)
 *       API oneapi::mkl::dft to perform 3-D Single Precision Real to Complex
-*       Fast-Fourier Transform on a SYCL device (Host, CPU, GPU).
+*       Fast-Fourier Transform on a SYCL device (CPU, GPU).
 *
 *       The supported floating point data types for data are:
 *           float
@@ -80,7 +80,7 @@ static void init_r(float *data, int M, int FWD_DIST,
                                          moda(n2, H2, N2) / N2+
                                          moda(n3, H3, N3) / N3;
                     int index = j*FWD_DIST + n3*S3 + n2*S2 + n1*S1;
-                    data[index] = factor * cosf(TWOPI * phase) / (N2*N1*N3);
+                    data[index] = factor * cosf(TWOPI * phase) / (N3*N2*N1);
                 }
             }
         }
@@ -88,7 +88,7 @@ static void init_r(float *data, int M, int FWD_DIST,
 }
 
 
-static int verify_c(float* data, int M, int BWD_DIST,
+static int verify_c(const float* data, int M, int BWD_DIST,
                     int N1, int N2, int N3, 
                     std::int64_t *cs, 
                     int H1, int H2, int H3)
@@ -163,7 +163,7 @@ static void init_c(float *data, int M, int BWD_DIST,
     }
 }
 
-static int verify_r(float *data, int M, int FWD_DIST,
+static int verify_r(const float *data, int M, int FWD_DIST,
                     int N1, int N2, int N3, 
                     std::int64_t *rs, 
                     int H1, int H2, int H3)
@@ -207,13 +207,13 @@ static int verify_r(float *data, int M, int FWD_DIST,
 }
 
 
-int run_dft_forward_example(cl::sycl::device &dev, 
+int run_dft_forward_example(sycl::device &dev, 
                             data_layout LAYOUT) {
     //
     // Initialize data for DFT
     //
     int N1 = 7, N2 = 13, N3 = 11;
-    int M = (LAYOUT == ROW_MAJOR ? 5 : 1); // can't do batch size > 1 if column-major, for now
+    int M = 5;
     
     
     // Arbitrary harmonic used to verify FFT
@@ -221,29 +221,31 @@ int run_dft_forward_example(cl::sycl::device &dev,
     // Strides describing data layout in real and conjugate-even domain
     std::int64_t rs[4];
     std::int64_t cs[4];
+    int FWD_DIST, BWD_DIST; 
     switch (LAYOUT)
     {
         case ROW_MAJOR:
             rs[0] = 0; rs[1] = N2*N1;         rs[2] = N1;           rs[3] = 1;  // row-major without padding 
             cs[0] = 0; cs[1] = N2*(N1/2 + 1); cs[2] = (N1/2 + 1);   cs[3] = 1;  // row-major without padding 
+            FWD_DIST = N1*N2*N3;
+            BWD_DIST = (N1/2 + 1)*N2*N3;
             break;
         case COL_MAJOR:
-            rs[0] = 0; rs[1] = 1;             rs[2] = N3;           rs[3] = N2*N3;  // column-major without padding 
-            cs[0] = 0; cs[1] = 1;             cs[2] = N3;           cs[3] = N2*N3;  // column-major without padding 
+            rs[0] = 0; rs[1] = M;             rs[2] = M*N3;         rs[3] = M*N2*N3;  // column-major without padding 
+            cs[0] = 0; cs[1] = M;             cs[2] = M*N3;         cs[3] = M*N2*N3;  // column-major without padding 
+            FWD_DIST = 1;
+            BWD_DIST = 1;
             break;
         default:
             throw std::invalid_argument("Invalid data layout. The only available options are ROW_MAJOR and COL_MAJOR");
     }
     
-    int FWD_DIST = N1*N2*N3;
-    int BWD_DIST = (N1/2 + 1)*N2*N3;
-    
     int result = FAILURE;
     int result_buffer = FAILURE;
     int result_usm = FAILURE;
 
-    float* x = (float*) mkl_malloc(FWD_DIST*M*sizeof(float), 64);
-    float* y = (float*) mkl_malloc(2*BWD_DIST*M*sizeof(float), 64);
+    float* x = (float*) mkl_malloc(M*N1*N2*N3*sizeof(float), 64);
+    float* y = (float*) mkl_malloc(M*2*(N1/2 + 1)*N2*N3*sizeof(float), 64);
     init_r(x, M, FWD_DIST, N1, N2, N3, rs, H1, H2, H3);
     
     //
@@ -251,11 +253,11 @@ int run_dft_forward_example(cl::sycl::device &dev,
     //
     try {
         // Catch asynchronous exceptions
-        auto exception_handler = [] (cl::sycl::exception_list exceptions) {
+        auto exception_handler = [] (sycl::exception_list exceptions) {
             for (std::exception_ptr const& e : exceptions) {
                 try {
                     std::rethrow_exception(e);
-                } catch(cl::sycl::exception const& e) {
+                } catch(sycl::exception const& e) {
                     std::cout << "Caught asynchronous SYCL exception:" << std::endl
                               << e.what() << std::endl;
                 }
@@ -263,16 +265,16 @@ int run_dft_forward_example(cl::sycl::device &dev,
         };
         
         // create execution queue with asynchronous error handling
-        cl::sycl::queue queue(dev, exception_handler);
+        sycl::queue queue(dev, exception_handler);
 
         // Setting up SYCL buffer and initialization
-        cl::sycl::buffer<float, 1> xBuffer(x, cl::sycl::range<1>(FWD_DIST*M));
-        cl::sycl::buffer<float, 1> yBuffer(y, cl::sycl::range<1>(2*BWD_DIST*M));
+        sycl::buffer<float, 1> xBuffer(x, sycl::range<1>(M*N1*N2*N3));
+        sycl::buffer<float, 1> yBuffer(y, sycl::range<1>(M*2*(N1/2 + 1)*N2*N3));
         xBuffer.set_write_back(false);
         yBuffer.set_write_back(false);
         // Setting up USM and initialization
-        float *x_usm = (float*) malloc_shared(FWD_DIST*M*sizeof(float), queue.get_device(), queue.get_context());
-        float *y_usm = (float*) malloc_shared(2*BWD_DIST*M*sizeof(float), queue.get_device(), queue.get_context());
+        float *x_usm = (float*) malloc_shared(M*N1*N2*N3*sizeof(float), queue.get_device(), queue.get_context());
+        float *y_usm = (float*) malloc_shared(M*2*(N1/2 + 1)*N2*N3*sizeof(float), queue.get_device(), queue.get_context());
         init_r(x_usm, M, FWD_DIST, N1, N2, N3, rs, H1, H2, H3);
 
         descriptor_t desc({N3, N2, N1});
@@ -288,7 +290,7 @@ int run_dft_forward_example(cl::sycl::device &dev,
         std::cout << "\tUsing SYCL buffers" << std::endl;
         oneapi::mkl::dft::compute_forward(desc, xBuffer, yBuffer);
 
-        auto yAcc = yBuffer.get_access<cl::sycl::access::mode::read>();
+        auto yAcc = yBuffer.get_host_access(sycl::read_only);
         result_buffer = verify_c(yAcc.get_pointer(), M, BWD_DIST, N1, N2, N3, cs, H1, H2, H3);
         
         // Using USM
@@ -303,7 +305,7 @@ int run_dft_forward_example(cl::sycl::device &dev,
         free(x_usm, queue.get_context());
         free(y_usm, queue.get_context());
     }
-    catch(cl::sycl::exception const& e) {
+    catch(sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during FFT" << std::endl;
         std::cout << "\t\t" << e.what() << std::endl;
         std::cout << "\t\tError code: " << get_error_code(e) << std::endl;
@@ -318,42 +320,44 @@ int run_dft_forward_example(cl::sycl::device &dev,
     return result;
 }
 
-int run_dft_backward_example(cl::sycl::device &dev, 
+int run_dft_backward_example(sycl::device &dev, 
                              data_layout LAYOUT) {
     //
     // Initialize data for DFT
     //
     int N1 = 7, N2 = 13, N3 = 11;
-    int M = (LAYOUT == ROW_MAJOR ? 5 : 1); // can't do batch size > 1 if column-major, for now
+    int M = 5;
     
     // Arbitrary harmonic used to verify FFT
     int H1 = 1, H2 = 2, H3 = 3;
     // Strides describing data layout in real and conjugate-even domain
     std::int64_t rs[4];
     std::int64_t cs[4];
+    int FWD_DIST, BWD_DIST; 
     switch (LAYOUT)
     {
         case ROW_MAJOR:
             rs[0] = 0; rs[1] = N2*N1;         rs[2] = N1;         rs[3] = 1; // row-major without padding 
             cs[0] = 0; cs[1] = N2*(N1/2 + 1); cs[2] = (N1/2 + 1); cs[3] = 1; // row-major without padding 
+            FWD_DIST = N1*N2*N3;
+            BWD_DIST = (N1/2 + 1)*N2*N3;
             break;
         case COL_MAJOR:
-            rs[0] = 0; rs[1] = 1;             rs[2] = N3;         rs[3] = N2*N3; // row-major without padding 
-            cs[0] = 0; cs[1] = 1;             cs[2] = N3;         cs[3] = N2*N3; // row-major without padding 
+            rs[0] = 0; rs[1] = M;             rs[2] = M*N3;       rs[3] = M*N2*N3; // column-major without padding
+            cs[0] = 0; cs[1] = M;             cs[2] = M*N3;       cs[3] = M*N2*N3; // column-major without padding
+            FWD_DIST = 1;
+            BWD_DIST = 1;
             break;
         default:
             throw std::invalid_argument("Invalid data layout. The only available options are ROW_MAJOR and COL_MAJOR");
     }
     
-    int FWD_DIST = N1*N2*N3;
-    int BWD_DIST = (N1/2 + 1)*N2*N3;
-    
     int result = FAILURE;
     int result_buffer = FAILURE;
     int result_usm = FAILURE;
 
-    float* x = (float*) mkl_malloc(FWD_DIST*M*sizeof(float), 64);
-    float* y = (float*) mkl_malloc(2*BWD_DIST*M*sizeof(float), 64);
+    float* x = (float*) mkl_malloc(M*N1*N2*N3*sizeof(float), 64);
+    float* y = (float*) mkl_malloc(M*2*(N1/2 + 1)*N2*N3*sizeof(float), 64);
     init_c(y, M, BWD_DIST, N1, N2, N3, cs, H1, H2, H3);
     
     //
@@ -361,11 +365,11 @@ int run_dft_backward_example(cl::sycl::device &dev,
     //
     try {
         // Catch asynchronous exceptions
-        auto exception_handler = [] (cl::sycl::exception_list exceptions) {
+        auto exception_handler = [] (sycl::exception_list exceptions) {
             for (std::exception_ptr const& e : exceptions) {
                 try {
                     std::rethrow_exception(e);
-                } catch(cl::sycl::exception const& e) {
+                } catch(sycl::exception const& e) {
                     std::cout << "Caught asynchronous SYCL exception:" << std::endl
                               << e.what() << std::endl;
                 }
@@ -373,16 +377,16 @@ int run_dft_backward_example(cl::sycl::device &dev,
         };
         
         // create execution queue with asynchronous error handling
-        cl::sycl::queue queue(dev, exception_handler);
+        sycl::queue queue(dev, exception_handler);
 
         // Setting up SYCL buffer and initialization
-        cl::sycl::buffer<float, 1> xBuffer(x, cl::sycl::range<1>(FWD_DIST*M));
-        cl::sycl::buffer<float, 1> yBuffer(y, cl::sycl::range<1>(2*BWD_DIST*M));
+        sycl::buffer<float, 1> xBuffer(x, sycl::range<1>(M*N1*N2*N3));
+        sycl::buffer<float, 1> yBuffer(y, sycl::range<1>(M*2*(N1/2 + 1)*N2*N3));
         xBuffer.set_write_back(false);
         yBuffer.set_write_back(false);
         // Setting up USM and initialization
-        float *x_usm = (float*) malloc_shared(FWD_DIST*M*sizeof(float), queue.get_device(), queue.get_context());
-        float *y_usm = (float*) malloc_shared(2*BWD_DIST*M*sizeof(float), queue.get_device(), queue.get_context());
+        float *x_usm = (float*) malloc_shared(M*N1*N2*N3*sizeof(float), queue.get_device(), queue.get_context());
+        float *y_usm = (float*) malloc_shared(M*2*(N1/2 + 1)*N2*N3*sizeof(float), queue.get_device(), queue.get_context());
         init_c(y_usm, M, BWD_DIST, N1, N2, N3, cs, H1, H2, H3);
         
 
@@ -399,7 +403,7 @@ int run_dft_backward_example(cl::sycl::device &dev,
         std::cout<<"\tUsing SYCL buffers"<<std::endl;
         oneapi::mkl::dft::compute_backward(desc, yBuffer, xBuffer);
 
-        auto xAcc = xBuffer.get_access<cl::sycl::access::mode::read>();
+        auto xAcc = xBuffer.get_host_access(sycl::read_only);
         result_buffer = verify_r(xAcc.get_pointer(), M, FWD_DIST, N1, N2, N3, rs, H1, H2, H3);
         
         // Using USM
@@ -415,7 +419,7 @@ int run_dft_backward_example(cl::sycl::device &dev,
         free(y_usm, queue.get_context());
         
     }
-    catch(cl::sycl::exception const& e) {
+    catch(sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during FFT" << std::endl;
         std::cout << "\t\t" << e.what() << std::endl;
         std::cout << "\t\tError code: " << get_error_code(e) << std::endl;
@@ -453,10 +457,9 @@ void print_example_banner() {
 // Main entry point for example.
 //
 // Dispatches to appropriate device types as set at build time with flag:
-// -DSYCL_DEVICES_host -- only runs host implementation
 // -DSYCL_DEVICES_cpu -- only runs SYCL CPU implementation
 // -DSYCL_DEVICES_gpu -- only runs SYCL GPU implementation
-// -DSYCL_DEVICES_all (default) -- runs on all: host, cpu and gpu devices
+// -DSYCL_DEVICES_all (default) -- runs on all: cpu and gpu devices
 //
 //
 int main() {
@@ -468,7 +471,7 @@ int main() {
 
     int returnCode = 0;
     for (auto it = list_of_devices.begin(); it != list_of_devices.end(); ++it) {
-        cl::sycl::device my_dev;
+        sycl::device my_dev;
         bool my_dev_is_found = false;
         get_sycl_device(my_dev, my_dev_is_found, *it);
 
@@ -506,5 +509,6 @@ int main() {
         }
     }
 
+    mkl_free_buffers();
     return returnCode;
 }
