@@ -16,6 +16,7 @@
 #define _MKL_RNG_DEVICE_PHILOX4X32X10_IMPL_HPP_
 
 #include <cstdint> // to use std::int32_t, std::uint64_t
+#include <utility> // std::pair
 
 namespace oneapi {
 namespace mkl {
@@ -45,56 +46,84 @@ union engine_state<oneapi::mkl::rng::device::philox4x32x10<VecSize>> {
 
 namespace philox4x32x10_impl {
 
-static inline std::uint64_t hi32(std::uint64_t t) {
-    return (t >> 32ull);
-}
-
-static inline std::uint64_t lo32(std::uint64_t t) {
-    return (t & 0xffffffff);
-}
-
 static inline void add128(std::uint32_t* a, std::uint64_t b) {
-    for (int i = 0; i < 4; i++) {
-        b += (std::uint64_t)a[i];
-        a[i] = lo32(b);
-        b = hi32(b);
+    std::uint32_t b_lo = static_cast<std::uint32_t>(b);
+    std::uint32_t b_hi = static_cast<std::uint32_t>(b >> 32);
+
+    a[0] += b_lo;
+    if (a[0] < b_lo) {
+        b_hi++;
     }
+    a[1] += b_hi;
+    if (a[1] >= b_hi) {
+        return;
+    }
+    if (++a[2]) {
+        return;
+    }
+    ++a[3];
+}
+
+static inline void add128_1(std::uint32_t* a) {
+    if (++a[0]) {
+        return;
+    }
+    if (++a[1]) {
+        return;
+    }
+    if (++a[2]) {
+        return;
+    }
+    ++a[3];
+}
+
+static inline std::pair<std::uint32_t, std::uint32_t> mul_hilo_32(std::uint32_t a,
+                                                                  std::uint32_t b) {
+    std::uint64_t res_64 = static_cast<std::uint64_t>(a) * static_cast<std::uint64_t>(b);
+    return std::make_pair(static_cast<std::uint32_t>(res_64),
+                          static_cast<std::uint32_t>(res_64 >> 32));
 }
 
 static inline void round(std::uint32_t* cnt, std::uint32_t* k) {
-    std::uint64_t tmp64_0, tmp64_2;
-    std::uint32_t K0, K1;
-    std::uint32_t L0, H0, L1, H1;
-    std::uint32_t C0, C1, C2, C3;
-    std::uint32_t R0, R1, R2, R3;
-    C0 = cnt[0];
-    C1 = cnt[1];
-    C2 = cnt[2];
-    C3 = cnt[3];
-    K0 = k[0];
-    K1 = k[1];
+    auto [L0, H0] = mul_hilo_32(0xD2511F53, cnt[0]);
+    auto [L1, H1] = mul_hilo_32(0xCD9E8D57, cnt[2]);
 
-    tmp64_0 = (std::uint64_t)C0 * 0xD2511F53ull;
-    L0 = lo32(tmp64_0);
-    H0 = hi32(tmp64_0);
-    tmp64_2 = (std::uint64_t)C2 * 0xCD9E8D57ull;
-    L1 = lo32(tmp64_2);
-    H1 = hi32(tmp64_2);
+    cnt[0] = H1 ^ cnt[1] ^ k[0];
+    cnt[1] = L1;
+    cnt[2] = H0 ^ cnt[3] ^ k[1];
+    cnt[3] = L0;
+}
 
-    R0 = H1 ^ C1 ^ K0;
-    R1 = L1;
-    R2 = H0 ^ C3 ^ K1;
-    R3 = L0;
-
-    K0 += 0x9E3779B9;
-    K1 += 0xBB67AE85;
-
-    cnt[0] = R0;
-    cnt[1] = R1;
-    cnt[2] = R2;
-    cnt[3] = R3;
-    k[0] = K0;
-    k[1] = K1;
+static inline void round_10(std::uint32_t* cnt, std::uint32_t* k) {
+    round(cnt, k); // 1
+    // increasing keys with philox4x32x10 constants
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 2
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 3
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 4
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 5
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 6
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 7
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 8
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 9
+    k[0] += 0x9E3779B9;
+    k[1] += 0xBB67AE85;
+    round(cnt, k); // 10
 }
 
 template <std::int32_t VecSize>
@@ -126,14 +155,12 @@ static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::phil
             counter[3] = state.counter[3];
             key[0] = state.key[0];
             key[1] = state.key[1];
-            for (int j = 0; j < 10; j++) {
-                round(counter, key);
-            }
+            round_10(counter, key);
             state.result[0] = counter[0];
             state.result[1] = counter[1];
             state.result[2] = counter[2];
             state.result[3] = counter[3];
-            add128(state.counter, 1);
+            add128_1(state.counter);
         }
     }
 }
@@ -142,15 +169,13 @@ template <std::int32_t VecSize>
 static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::philox4x32x10<VecSize>,
                                                   device_type::generic>& state,
                               std::uint64_t n, const std::uint64_t* num_to_skip_ptr) {
-    std::uint64_t uint_max = 0xFFFFFFFFFFFFFFFF;
-    std::uint32_t counter[4];
-    std::uint32_t key[4];
+    constexpr std::uint64_t uint_max = 0xFFFFFFFFFFFFFFFF;
     std::uint64_t post_buffer, pre_buffer;
     std::int32_t num_elements = 0;
     std::int32_t remained_counter;
     std::uint64_t tmp_skip_array[3] = { 0, 0, 0 };
 
-    for (int i = 0; (i < 3) && (i < n); i++) {
+    for (std::uint64_t i = 0; (i < 3) && (i < n); i++) {
         tmp_skip_array[i] = num_to_skip_ptr[i];
         if (tmp_skip_array[i]) {
             num_elements = i + 1;
@@ -161,9 +186,11 @@ static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::phil
         return;
     }
     if ((num_elements == 1) && (tmp_skip_array[0] <= state.part)) {
-        state.part -= (std::uint32_t)tmp_skip_array[0];
+        state.part -= static_cast<std::uint32_t>(tmp_skip_array[0]);
         return;
     }
+    std::uint32_t counter[4];
+    std::uint32_t key[2];
 
     if ((tmp_skip_array[0] - state.part) <= tmp_skip_array[0]) {
         tmp_skip_array[0] = tmp_skip_array[0] - state.part;
@@ -182,7 +209,7 @@ static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::phil
 
     post_buffer = 0;
 
-    remained_counter = (std::uint32_t)(tmp_skip_array[0] % 4);
+    remained_counter = static_cast<std::uint32_t>(tmp_skip_array[0] % 4);
 
     for (int i = num_elements - 1; i >= 0; i--) {
         pre_buffer = (tmp_skip_array[i] << 62);
@@ -205,16 +232,16 @@ static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::phil
 
     counter64[1] += tmp_skip_array[1];
 
-    counter[0] = lo32(counter64[0]);
-    counter[1] = hi32(counter64[0]);
-    counter[2] = lo32(counter64[1]);
-    counter[3] = hi32(counter64[1]);
+    counter[0] = static_cast<std::uint32_t>(counter64[0]);
+    counter[1] = static_cast<std::uint32_t>(counter64[0] >> 32);
+    counter[2] = static_cast<std::uint32_t>(counter64[1]);
+    counter[3] = static_cast<std::uint32_t>(counter64[1] >> 32);
 
     key[0] = state.key[0];
     key[1] = state.key[1];
-    for (int i = 0; i < 10; i++) {
-        round(counter, key);
-    }
+
+    round_10(counter, key);
+
     state.result[0] = counter[0];
     state.result[1] = counter[1];
     state.result[2] = counter[2];
@@ -226,24 +253,24 @@ static inline void skip_ahead(engine_state_device<oneapi::mkl::rng::device::phil
         counter64[1]++;
     }
 
-    state.counter[0] = lo32(counter64[0]);
-    state.counter[1] = hi32(counter64[0]);
-    state.counter[2] = lo32(counter64[1]);
-    state.counter[3] = hi32(counter64[1]);
+    state.counter[0] = static_cast<std::uint32_t>(counter64[0]);
+    state.counter[1] = static_cast<std::uint32_t>(counter64[0] >> 32);
+    state.counter[2] = static_cast<std::uint32_t>(counter64[1]);
+    state.counter[3] = static_cast<std::uint32_t>(counter64[1] >> 32);
 }
 
 template <std::int32_t VecSize>
 static inline void init(engine_state_device<oneapi::mkl::rng::device::philox4x32x10<VecSize>,
                                             device_type::generic>& state,
                         std::uint64_t n, const std::uint64_t* seed_ptr, std::uint64_t offset) {
-    state.key[0] = lo32(seed_ptr[0]);
-    state.key[1] = hi32(seed_ptr[0]);
+    state.key[0] = static_cast<std::uint32_t>(seed_ptr[0]);
+    state.key[1] = static_cast<std::uint32_t>(seed_ptr[0] >> 32);
 
-    state.counter[0] = (n >= 2 ? lo32(seed_ptr[1]) : 0);
-    state.counter[1] = (n >= 2 ? hi32(seed_ptr[1]) : 0);
+    state.counter[0] = (n >= 2 ? static_cast<std::uint32_t>(seed_ptr[1]) : 0);
+    state.counter[1] = (n >= 2 ? static_cast<std::uint32_t>(seed_ptr[1] >> 32) : 0);
 
-    state.counter[2] = (n >= 3 ? lo32(seed_ptr[2]) : 0);
-    state.counter[3] = (n >= 3 ? hi32(seed_ptr[2]) : 0);
+    state.counter[2] = (n >= 3 ? static_cast<std::uint32_t>(seed_ptr[2]) : 0);
+    state.counter[3] = (n >= 3 ? static_cast<std::uint32_t>(seed_ptr[2] >> 32) : 0);
 
     state.part = 0;
     state.result[0] = 0;
@@ -258,14 +285,14 @@ static inline void init(engine_state_device<oneapi::mkl::rng::device::philox4x32
                                             device_type::generic>& state,
                         std::uint64_t n, const std::uint64_t* seed_ptr, std::uint64_t n_offset,
                         const std::uint64_t* offset_ptr) {
-    state.key[0] = lo32(seed_ptr[0]);
-    state.key[1] = hi32(seed_ptr[0]);
+    state.key[0] = static_cast<std::uint32_t>(seed_ptr[0]);
+    state.key[1] = static_cast<std::uint32_t>(seed_ptr[0] >> 32);
 
-    state.counter[0] = (n >= 2 ? lo32(seed_ptr[1]) : 0);
-    state.counter[1] = (n >= 2 ? hi32(seed_ptr[1]) : 0);
+    state.counter[0] = (n >= 2 ? static_cast<std::uint32_t>(seed_ptr[1]) : 0);
+    state.counter[1] = (n >= 2 ? static_cast<std::uint32_t>(seed_ptr[1] >> 32) : 0);
 
-    state.counter[2] = (n >= 3 ? lo32(seed_ptr[2]) : 0);
-    state.counter[3] = (n >= 3 ? hi32(seed_ptr[2]) : 0);
+    state.counter[2] = (n >= 3 ? static_cast<std::uint32_t>(seed_ptr[2]) : 0);
+    state.counter[3] = (n >= 3 ? static_cast<std::uint32_t>(seed_ptr[2] >> 32) : 0);
 
     state.part = 0;
     state.result[0] = 0;
@@ -321,6 +348,7 @@ static inline void store(
 
 // for VecSize > 4
 template <std::int32_t VecSize>
+__attribute__((always_inline))
 static inline sycl::vec<std::uint32_t, VecSize> generate_full(
     engine_state_device<oneapi::mkl::rng::device::philox4x32x10<VecSize>, device_type::generic>&
         state) {
@@ -328,9 +356,6 @@ static inline sycl::vec<std::uint32_t, VecSize> generate_full(
     sycl::vec<std::uint32_t, VecSize> res;
 
     std::uint32_t counter[4];
-    std::uint32_t cntTmp[4];
-    std::uint32_t key[2];
-    std::uint32_t keyTmp[2];
 
     int i = 0;
     int part = (int)state.part;
@@ -346,26 +371,25 @@ static inline sycl::vec<std::uint32_t, VecSize> generate_full(
     counter[1] = state.counter[1];
     counter[2] = state.counter[2];
     counter[3] = state.counter[3];
-    key[0] = state.key[0];
-    key[1] = state.key[1];
+
+    std::uint32_t cntTmp[4];
+    std::uint32_t keyTmp[2];
     for (; i < num_elements; i += 4) {
         cntTmp[0] = counter[0];
         cntTmp[1] = counter[1];
         cntTmp[2] = counter[2];
         cntTmp[3] = counter[3];
 
-        keyTmp[0] = key[0];
-        keyTmp[1] = key[1];
+        keyTmp[0] = state.key[0];
+        keyTmp[1] = state.key[1];
 
-        for (int j = 0; j < 10; j++) {
-            round(cntTmp, keyTmp);
-        }
+        round_10(cntTmp, keyTmp);
 
         if (i + 4 <= num_elements) {
             for (int j = 0; j < 4; j++) {
                 res[i + j] = cntTmp[j];
             }
-            add128(counter, 1);
+            add128_1(counter);
         }
         else {
             // here if last iteration
@@ -380,6 +404,7 @@ static inline sycl::vec<std::uint32_t, VecSize> generate_full(
 
 // for VecSize <= 4
 template <std::int32_t VecSize>
+__attribute__((always_inline))
 static inline sycl::vec<std::uint32_t, VecSize> generate_small(
     engine_state_device<oneapi::mkl::rng::device::philox4x32x10<VecSize>, device_type::generic>&
         state) {
@@ -405,9 +430,8 @@ static inline sycl::vec<std::uint32_t, VecSize> generate_small(
     counter[3] = state.counter[3];
     key[0] = state.key[0];
     key[1] = state.key[1];
-    for (int j = 0; j < 10; j++) {
-        round(counter, key);
-    }
+
+    round_10(counter, key);
 
     for (int j = 0; i < num_elements; i++, j++) {
         res[i] = counter[j];
@@ -418,6 +442,7 @@ static inline sycl::vec<std::uint32_t, VecSize> generate_small(
 }
 
 template <int VecSize>
+__attribute__((always_inline))
 static inline std::uint32_t generate_single(
     engine_state_device<oneapi::mkl::rng::device::philox4x32x10<VecSize>, device_type::generic>&
         state) {
@@ -426,7 +451,7 @@ static inline std::uint32_t generate_single(
     std::uint32_t counter[4];
     std::uint32_t key[2];
 
-    std::int32_t part = (std::int32_t)state.part;
+    std::int32_t part = static_cast<std::int32_t>(state.part);
     if (part != 0) {
         res = state.result[3 - (--part)];
         skip_ahead(state, 1);
@@ -438,9 +463,8 @@ static inline std::uint32_t generate_single(
     counter[3] = state.counter[3];
     key[0] = state.key[0];
     key[1] = state.key[1];
-    for (int j = 0; j < 10; j++) {
-        round(counter, key);
-    }
+
+    round_10(counter, key);
 
     res = counter[0];
 
@@ -540,7 +564,8 @@ protected:
     }
 
     template <typename RealType>
-    auto generate(RealType a, RealType b) ->
+    __attribute__((always_inline))
+    inline auto generate(RealType a, RealType b) ->
         typename std::conditional<VecSize == 1, RealType, sycl::vec<RealType, VecSize>>::type {
         sycl::vec<RealType, VecSize> res;
         sycl::vec<std::uint32_t, VecSize> res_uint;
@@ -557,12 +582,13 @@ protected:
             res_uint = philox4x32x10_impl::generate_small(this->state_.generic_state);
         }
         for (int i = 0; i < VecSize; i++) {
-            res[i] = (RealType)((std::int32_t)res_uint[i]) * c1 + a1;
+            res[i] = static_cast<RealType>(static_cast<std::int32_t>(res_uint[i])) * c1 + a1;
         }
         return res;
     }
 
-    auto generate() -> typename std::conditional<VecSize == 1, std::uint32_t,
+    __attribute__((always_inline))
+    inline auto generate() -> typename std::conditional<VecSize == 1, std::uint32_t,
                                                  sycl::vec<std::uint32_t, VecSize>>::type {
         if constexpr (VecSize > 4) {
             return philox4x32x10_impl::generate_full(this->state_.generic_state);
@@ -570,6 +596,38 @@ protected:
         return philox4x32x10_impl::generate_small(this->state_.generic_state);
     }
 
+    template <typename UIntType>
+    __attribute__((always_inline))
+    inline auto generate_uniform_bits() ->
+        typename std::conditional<VecSize == 1, UIntType,
+                                                 sycl::vec<UIntType, VecSize>>::type {
+        if constexpr (std::is_same<UIntType, std::uint32_t>::value) {
+            return generate();
+        } else {
+            auto uni_res1 = generate();
+            auto uni_res2 = generate();
+
+            if constexpr (VecSize == 1) {
+                return (static_cast<std::uint64_t>(uni_res2) << 32) + uni_res1;
+            } else {
+                sycl::vec<std::uint64_t, VecSize> vec_out;
+
+                if constexpr (VecSize != 3) {
+                    for (int i = 0; i < VecSize / 2; i++) {
+                        vec_out[i] = (static_cast<std::uint64_t>(uni_res1[2 * i + 1]) << 32) + uni_res1[2 * i];
+                        vec_out[i + VecSize / 2] = (static_cast<std::uint64_t>(uni_res2[2 * i + 1]) << 32) + uni_res2[2 * i];
+                    }
+                } else {
+                    vec_out[0] = (static_cast<std::uint64_t>(uni_res1[1]) << 32) + uni_res1[0];
+                    vec_out[1] = (static_cast<std::uint64_t>(uni_res2[0]) << 32) + uni_res1[2];
+                    vec_out[2] = (static_cast<std::uint64_t>(uni_res2[2]) << 32) + uni_res2[1];
+                }
+
+                return vec_out;
+            }
+        }
+    }
+
     template <typename RealType>
     RealType generate_single(RealType a, RealType b) {
         RealType res;
@@ -582,15 +640,29 @@ protected:
 
         res_uint = philox4x32x10_impl::generate_single(this->state_.generic_state);
 
-        res = (RealType)((std::int32_t)res_uint) * c1 + a1;
+        res = static_cast<RealType>(static_cast<std::int32_t>(res_uint)) * c1 + a1;
 
         return res;
     }
 
-    std::uint32_t generate_single() {
+    __attribute__((always_inline))
+    inline std::uint32_t generate_single() {
         return philox4x32x10_impl::generate_single(this->state_.generic_state);
     }
 
+    template <typename UIntType>
+    __attribute__((always_inline))
+    inline auto generate_single_uniform_bits() {
+        if constexpr (std::is_same<UIntType, std::uint32_t>::value) {
+            return philox4x32x10_impl::generate_single(this->state_.generic_state);
+        } else {
+            auto uni_res1 = philox4x32x10_impl::generate_single(this->state_.generic_state);
+            auto uni_res2 = philox4x32x10_impl::generate_single(this->state_.generic_state);
+
+            return (static_cast<std::uint64_t>(uni_res2) << 32) + uni_res1;
+        }
+    }
+
     void skip_ahead(std::uint64_t num_to_skip) {
         detail::philox4x32x10_impl::skip_ahead(this->state_.generic_state, num_to_skip);
     }
