@@ -66,7 +66,7 @@ void print_output(sycl::buffer<Type>& r, int n_print) {
 
 // function to compare theoretical moments and sample moments
 template<typename Type>
-bool compare_moments(Type* r, std::size_t size, double tM, double tD, double tQ) {
+bool compare_moments(const Type* r, std::size_t size, double tM, double tD, double tQ) {
     // sample moments
     double sum = 0.0;
     double sum2 = 0.0;
@@ -101,7 +101,7 @@ bool compare_moments(Type* r, std::size_t size, double tM, double tD, double tQ)
 
 // function to compare theoretical moments and sample moments for multidimensional gaussian distribution
 template<typename Type>
-bool compare_moments_gaussian_mv(Type* r, std::size_t size, Type* mean_theory, Type* variance_theory, std::uint32_t dimen) {
+bool compare_moments_gaussian_mv(const Type* r, std::size_t size, Type* mean_theory, Type* variance_theory, std::uint32_t dimen) {
     // vectors for computed moments
     std::vector<Type> mean_generated(dimen, 0.0);
     std::vector<Type> var_generated(dimen, 0.0); // elements are stored in layout::packed format
@@ -144,7 +144,7 @@ bool compare_moments_gaussian_mv(Type* r, std::size_t size, Type* mean_theory, T
 // and compare them with sample moments
 template<typename Type, typename Method>
 std::enable_if_t<!std::is_integral_v<Type>, bool>
-check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type, Method>& distr) {
+check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type b = distr.b();
@@ -159,7 +159,7 @@ check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type
 
 template<typename Type, typename Method>
 std::enable_if_t<std::is_integral_v<Type>, bool>
-check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type, Method>& distr) {
+check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type b = distr.b();
@@ -173,7 +173,7 @@ check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::uniform<Type
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gaussian<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::gaussian<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.mean();
     Type sigma = distr.stddev();
@@ -187,7 +187,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gaussia
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size,
+bool check_statistics(const Type* r, std::size_t size,
     const oneapi::mkl::rng::gaussian_mv<Type, oneapi::mkl::rng::layout::packed, Method>& distr) {
     // quantity of dimensions
     std::uint32_t dimen = distr.dimen();
@@ -229,7 +229,7 @@ bool check_statistics(Type* r, std::size_t size,
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size,
+bool check_statistics(const Type* r, std::size_t size,
     const oneapi::mkl::rng::gaussian_mv<Type, oneapi::mkl::rng::layout::full, Method>& distr) {
     // quantity of dimensions
     std::uint32_t dimen = distr.dimen();
@@ -264,7 +264,7 @@ bool check_statistics(Type* r, std::size_t size,
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size,
+bool check_statistics(const Type* r, std::size_t size,
     const oneapi::mkl::rng::gaussian_mv<Type, oneapi::mkl::rng::layout::diagonal, Method>& distr) {
     // quantity of dimensions
     std::uint32_t dimen = distr.dimen();
@@ -282,7 +282,7 @@ bool check_statistics(Type* r, std::size_t size,
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::lognormal<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::lognormal<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.m();
     Type b = distr.displ();
@@ -299,7 +299,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::lognorm
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::beta<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::beta<Type, Method>& distr) {
     double tM, tD, tQ;
     double b, c, d, e, e2, b2, sum_pq;
     Type p = distr.p();
@@ -324,7 +324,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::beta<Ty
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::chi_square<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::chi_square<Type, Method>& distr) {
     double tM, tD, tQ;
     std::int32_t n = distr.n();
 
@@ -337,7 +337,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::chi_squ
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::exponential<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::exponential<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type beta = distr.beta();
@@ -351,7 +351,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::exponen
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gamma<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::gamma<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type alpha = distr.alpha();
@@ -366,7 +366,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gamma<T
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gumbel<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::gumbel<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type beta = distr.b();
@@ -380,7 +380,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::gumbel<
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::laplace<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::laplace<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type beta = distr.b();
@@ -394,7 +394,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::laplace
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::rayleigh<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::rayleigh<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type beta = distr.b();
@@ -408,7 +408,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::rayleig
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::weibull<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::weibull<Type, Method>& distr) {
     double tM, tD, tQ;
     Type a = distr.a();
     Type beta = distr.beta();
@@ -422,7 +422,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::weibull
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::bernoulli<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::bernoulli<Type, Method>& distr) {
     double tM, tD, tQ;
     double p = distr.p();
 
@@ -435,7 +435,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::bernoul
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::binomial<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::binomial<Type, Method>& distr) {
     double tM, tD, tQ;
     double p = distr.p();
     std::int32_t ntrial = distr.ntrial();
@@ -449,7 +449,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::binomia
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::geometric<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::geometric<Type, Method>& distr) {
     double tM, tD, tQ;
     double p = distr.p();
 
@@ -462,7 +462,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::geometr
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::hypergeometric<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::hypergeometric<Type, Method>& distr) {
     double tM, tD, tQ;
     double K, L2, L3, L4, L5, L6, KL, KL4, S2, S3, S4, M2, M3, M4;
     std::int32_t l = distr.l();
@@ -496,7 +496,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::hyperge
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::multinomial<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::multinomial<Type, Method>& distr) {
     double tM,tD,tQ,tD2;
     double sM,sD;
     double sum, sum2;
@@ -537,7 +537,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::multino
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::negative_binomial<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::negative_binomial<Type, Method>& distr) {
     double tM, tD, tQ;
     double p = distr.p();
     double a = distr.a();
@@ -551,7 +551,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::negativ
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::poisson<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::poisson<Type, Method>& distr) {
     double tM, tD, tQ;
     double lambda = distr.lambda();
 
@@ -564,7 +564,7 @@ bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::poisson
 }
 
 template<typename Type, typename Method>
-bool check_statistics(Type* r, std::size_t size, const oneapi::mkl::rng::poisson_v<Type, Method>& distr) {
+bool check_statistics(const Type* r, std::size_t size, const oneapi::mkl::rng::poisson_v<Type, Method>& distr) {
     double tM, tD, tQ;
     std::vector<double> lambda = distr.lambda();
 
