@@ -17,8 +17,8 @@
 *  Content:
 *       This example demonstrates use of oneAPI Math Kernel Library (oneMKL)
 *       DPCPP USM API oneapi::mkl::sparse::matmat to perform general
-*       sparse matrix-sparse matrix multiplication on a SYCL device (Host, CPU, GPU)
-*       but only calculate the sparsity pattern of C, not fill it with values..
+*       sparse matrix-sparse matrix multiplication on a SYCL device (CPU, GPU)
+*       but only calculate the sparsity pattern of C, not fill it with values.
 *
 *           C = op(A) * op(B)
 *
@@ -28,7 +28,7 @@
 *
 *       It uses the full control API usage model where the user handles both memory
 *       allocation for the final C Matrix, and allocation of any temporary
-*       workspaces along the way.
+*       workspaces along the way. All matrices use USM shared allocation.
 *
 *       The supported floating point data types for matmat matrix data are:
 *           float
@@ -66,7 +66,7 @@
 // square of A.
 //
 template <typename fpType, typename intType>
-int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev)
+int run_sparse_matrix_sparse_matrix_multiply_example(const sycl::device &dev)
 {
     // Initialize data for Sparse Matrix - Sparse Matrix Multiply
     oneapi::mkl::transpose opA = oneapi::mkl::transpose::trans;
@@ -81,12 +81,12 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     oneapi::mkl::index_base c_index = oneapi::mkl::index_base::zero;
 
     // Catch asynchronous exceptions
-    auto exception_handler = [](cl::sycl::exception_list exceptions) {
+    auto exception_handler = [](sycl::exception_list exceptions) {
         for (std::exception_ptr const &e : exceptions) {
             try {
                 std::rethrow_exception(e);
             }
-            catch (cl::sycl::exception const &e) {
+            catch (sycl::exception const &e) {
                 std::cout << "Caught asynchronous SYCL "
                              "exception during sparse::gemv:\n"
                           << e.what() << std::endl;
@@ -95,8 +95,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     };
 
     // create execution main_queue
-    cl::sycl::queue main_queue(dev, exception_handler);
-    cl::sycl::context cxt = main_queue.get_context();
+    sycl::queue main_queue(dev, exception_handler);
+    sycl::context cxt = main_queue.get_context();
 
     //
     // set up dimensions of matrix products
@@ -124,88 +124,46 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     //
     // setup A data locally in CSR format
     //
-    std::vector<intType, mkl_allocator<intType, 64>> ia;
-    std::vector<intType, mkl_allocator<intType, 64>> ja;
-    std::vector<fpType, mkl_allocator<fpType, 64>> a;
+    intType *a_rowptr = sycl::malloc_shared<intType>(a_nrows + 1, main_queue);
+    intType *a_colind = sycl::malloc_shared<intType>(27 * a_nrows, main_queue);
+    fpType *a_values  = sycl::malloc_shared<fpType>(27 * a_nrows, main_queue);
 
-    ia.resize(a_nrows + 1);
-    ja.resize(27 * a_nrows);
-    a.resize(27 * a_nrows);
-
-    intType a_ind = a_index == oneapi::mkl::index_base::zero ? 0 : 1;
-    generate_sparse_matrix<fpType, intType>(size, ia, ja, a, a_ind);
-    a_nnz = ia[a_nrows] - a_ind;
-
-    // USM arrays for A matrix
-    intType *a_rowptr, *a_colind;
-    fpType *a_val;
-
-    a_rowptr = (intType *)malloc_shared((a_nrows + 1) * sizeof(intType), dev, cxt);
-    a_colind = (intType *)malloc_shared((a_nnz) * sizeof(intType), dev, cxt);
-    a_val    = (fpType *)malloc_shared((a_nnz) * sizeof(fpType), dev, cxt);
-
-    if (!a_rowptr || !a_colind || !a_val)
+    if (!a_rowptr || !a_colind || !a_values)
         throw std::runtime_error("Failed to allocate USM memory");
 
     int_ptr_vec.push_back(a_rowptr);
     int_ptr_vec.push_back(a_colind);
-    fp_ptr_vec.push_back(a_val);
+    fp_ptr_vec.push_back(a_values);
 
-    // copy data to USM arrays
-    for (intType i = 0; i < a_nrows + 1; ++i) {
-        a_rowptr[i] = ia[i];
-    }
-    for (intType i = 0; i < a_nnz; ++i) {
-        a_colind[i] = ja[i];
-        a_val[i]    = a[i];
-    }
+    intType a_ind = a_index == oneapi::mkl::index_base::zero ? 0 : 1;
+    generate_sparse_matrix<fpType, intType>(size, a_rowptr, a_colind, a_values, a_ind);
+    a_nnz = a_rowptr[a_nrows] - a_ind;
 
     //
     // setup B data locally in CSR format
     //
-    std::vector<intType, mkl_allocator<intType, 64>> ib;
-    std::vector<intType, mkl_allocator<intType, 64>> jb;
-    std::vector<fpType, mkl_allocator<fpType, 64>> b;
+    intType *b_rowptr = sycl::malloc_shared<intType>(b_nrows + 1, main_queue);
+    intType *b_colind = sycl::malloc_shared<intType>(27 * b_nrows, main_queue);
+    fpType *b_values  = sycl::malloc_shared<fpType>(27 * b_nrows, main_queue);
 
-    ib.resize(b_nrows + 1);
-    jb.resize(27 * b_nrows);
-    b.resize(27 * b_nrows);
-
-    intType b_ind = b_index == oneapi::mkl::index_base::zero ? 0 : 1;
-    generate_sparse_matrix<fpType, intType>(size, ib, jb, b, b_ind);
-    b_nnz = ib[b_nrows] - b_ind;
-
-    // USM arrays for B matrix
-    intType *b_rowptr, *b_colind;
-    fpType *b_val;
-
-    b_rowptr = (intType *)malloc_shared((b_nrows + 1) * sizeof(intType), dev, cxt);
-    b_colind = (intType *)malloc_shared((b_nnz) * sizeof(intType), dev, cxt);
-    b_val    = (fpType *)malloc_shared((b_nnz) * sizeof(fpType), dev, cxt);
-
-    if (!b_rowptr || !b_colind || !b_val)
+    if (!b_rowptr || !b_colind || !b_values)
         throw std::runtime_error("Failed to allocate USM memory");
 
     int_ptr_vec.push_back(b_rowptr);
     int_ptr_vec.push_back(b_colind);
-    fp_ptr_vec.push_back(b_val);
+    fp_ptr_vec.push_back(b_values);
 
-    // copy data to USM arrays
-    for (intType i = 0; i < b_nrows + 1; ++i) {
-        b_rowptr[i] = ib[i];
-    }
-    for (intType i = 0; i < b_nnz; ++i) {
-        b_colind[i] = jb[i];
-        b_val[i]    = b[i];
-    }
+    intType b_ind = b_index == oneapi::mkl::index_base::zero ? 0 : 1;
+    generate_sparse_matrix<fpType, intType>(size, b_rowptr, b_colind, b_values, b_ind);
+    b_nnz = b_rowptr[b_nrows] - b_ind;
 
     //
     // setup C data locally in CSR format
     // note: we don't know c_nnz so can only prepare ic at this point
     //
-    intType *c_rowptr = nullptr, *c_colind = nullptr;
-
-    c_rowptr = (intType *)malloc_shared((c_nrows + 1) * sizeof(intType), dev, cxt);
+    intType c_ind = c_index == oneapi::mkl::index_base::zero ? 0 : 1;
+    intType *c_colind = nullptr;
+    intType *c_rowptr = sycl::malloc_shared<intType>(c_nrows + 1, main_queue);
 
     if (!c_rowptr)
         throw std::runtime_error("Failed to allocate USM memory");
@@ -218,6 +176,9 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     std::int64_t *sizeTempBuffer = nullptr, *sizeTempBuffer2 = nullptr, *c_nnz = nullptr;
     void *tempBuffer = nullptr, *tempBuffer2 = nullptr;
 
+    //
+    // Execute Matrix Multiply
+    //
 
     std::cout << "\n\t\tsparse::matmat parameters:\n";
     std::cout << "\t\t\topA = " << opA << std::endl;
@@ -235,15 +196,11 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     std::cout << "\t\t\tB_index = " << b_index << std::endl;
     std::cout << "\t\t\tC_index = " << c_index << std::endl;
 
-    //
-    // Execute Matrix Multiply
-    //
+    oneapi::mkl::sparse::matrix_handle_t A = nullptr;
+    oneapi::mkl::sparse::matrix_handle_t B = nullptr;
+    oneapi::mkl::sparse::matrix_handle_t C = nullptr;
 
-    oneapi::mkl::sparse::matrix_handle_t A;
-    oneapi::mkl::sparse::matrix_handle_t B;
-    oneapi::mkl::sparse::matrix_handle_t C;
-
-    oneapi::mkl::sparse::matmat_descr_t descr = NULL;
+    oneapi::mkl::sparse::matmat_descr_t descr = nullptr;
     oneapi::mkl::sparse::matmat_request req;
 
     try {
@@ -251,14 +208,14 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         oneapi::mkl::sparse::init_matrix_handle(&B);
         oneapi::mkl::sparse::init_matrix_handle(&C);
 
-        oneapi::mkl::sparse::set_csr_data(A, a_nrows, a_ncols, a_index, a_rowptr, a_colind, a_val);
-        oneapi::mkl::sparse::set_csr_data(B, b_nrows, b_ncols, b_index, b_rowptr, b_colind, b_val);
+        auto ev_setA = oneapi::mkl::sparse::set_csr_data(main_queue, A, a_nrows, a_ncols, a_index, a_rowptr, a_colind, a_values, {});
+        auto ev_setB = oneapi::mkl::sparse::set_csr_data(main_queue, B, b_nrows, b_ncols, b_index, b_rowptr, b_colind, b_values, {});
 
         //
         // only c_rowptr exists at this point in process so pass in nullptrs
         //
-        oneapi::mkl::sparse::set_csr_data(C, c_nrows, c_ncols, c_index, c_rowptr,
-                                          (intType *)nullptr, (fpType *)nullptr);
+        auto ev_setC = oneapi::mkl::sparse::set_csr_data(main_queue, C, c_nrows, c_ncols, c_index, c_rowptr,
+                                          (intType *)nullptr, (fpType *)nullptr, {});
 
         //
         // initialize the matmat descriptor
@@ -272,25 +229,25 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 1.1
         //   query for size of work_estimation temp buffer
-        req            = oneapi::mkl::sparse::matmat_request::get_work_estimation_buf_size;
-        sizeTempBuffer = (std::int64_t *)malloc_shared(sizeof(std::int64_t), dev, cxt);
+        req = oneapi::mkl::sparse::matmat_request::get_work_estimation_buf_size;
+        sizeTempBuffer = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!sizeTempBuffer)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(sizeTempBuffer);
 
         auto ev1_1 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer,
-                                                 nullptr, {});
+                                                 nullptr, {ev_setA, ev_setB, ev_setC});
 
         // Step 1.2
         //   allocate temp buffer for work_estimation
         ev1_1.wait();
-        void *tempBuffer = malloc_shared(sizeTempBuffer[0] * sizeof(std::uint8_t), dev, cxt);
+        void *tempBuffer = sycl::malloc_device(sizeTempBuffer[0] * sizeof(std::uint8_t), main_queue);
         if (!tempBuffer)
             throw std::runtime_error("Failed to allocate USM memory");
         void_ptr_vec.push_back(tempBuffer);
 
         // Step 1.3  do work_estimation
-        req        = oneapi::mkl::sparse::matmat_request::work_estimation;
+        req = oneapi::mkl::sparse::matmat_request::work_estimation;
         auto ev1_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer,
                                                  tempBuffer, {ev1_1});
 
@@ -299,8 +256,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         //
 
         // Step 2.1 query size of compute_structure temp buffer
-        req             = oneapi::mkl::sparse::matmat_request::get_compute_structure_buf_size;
-        sizeTempBuffer2 = (std::int64_t *)malloc_shared(sizeof(std::int64_t), dev, cxt);
+        req = oneapi::mkl::sparse::matmat_request::get_compute_structure_buf_size;
+        sizeTempBuffer2 = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!sizeTempBuffer2)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(sizeTempBuffer2);
@@ -309,13 +266,13 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 2.2 allocate temp buffer for compute_structure
         ev2_1.wait();
-        void *tempBuffer2 = malloc_shared(sizeTempBuffer2[0] * sizeof(std::uint8_t), dev, cxt);
+        void *tempBuffer2 = sycl::malloc_device(sizeTempBuffer2[0] * sizeof(std::uint8_t), main_queue);
         if (!tempBuffer2)
             throw std::runtime_error("Failed to allocate USM memory");
         void_ptr_vec.push_back(tempBuffer2);
 
         // Step 2.3 do compute
-        req        = oneapi::mkl::sparse::matmat_request::compute_structure;
+        req = oneapi::mkl::sparse::matmat_request::compute_structure;
         auto ev2_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer2,
                                                  tempBuffer2, {ev2_1});
 
@@ -324,8 +281,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         //
 
         // Step 3.1  get nnz
-        req                 = oneapi::mkl::sparse::matmat_request::get_nnz;
-        std::int64_t *c_nnz = (std::int64_t *)(malloc_shared(1 * sizeof(std::int64_t), dev, cxt));
+        req = oneapi::mkl::sparse::matmat_request::get_nnz;
+        std::int64_t *c_nnz = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!c_nnz)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(c_nnz);
@@ -334,22 +291,25 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 3.2  allocate final c matrix arrays
         ev3_1.wait();
-        c_colind = (intType *)malloc_shared(c_nnz[0] * sizeof(intType), dev, cxt);
+        c_colind = sycl::malloc_shared<intType>(c_nnz[0], main_queue);
         if (!c_colind) {
-            throw std::bad_alloc();
+            throw std::runtime_error("Failed to allocate USM memory");
         }
         int_ptr_vec.push_back(c_colind);
-        oneapi::mkl::sparse::set_csr_data(C, c_nrows, c_ncols, c_index, c_rowptr, c_colind,
-                                          (fpType *)nullptr);
+        ev_setC = oneapi::mkl::sparse::set_csr_data(main_queue, C, c_nrows, c_ncols, c_index, c_rowptr, c_colind,
+                                          (fpType *)nullptr, {ev3_1});
 
         // Step 3.3  finalize into C matrix
-        req        = oneapi::mkl::sparse::matmat_request::finalize_structure;
+        req = oneapi::mkl::sparse::matmat_request::finalize_structure;
         auto ev3_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, nullptr, nullptr,
-                                                 {ev3_1});
+                                                 {ev_setC});
+
+        // Sort C matrix output if desired
+        auto ev_sort = oneapi::mkl::sparse::sort_matrix(main_queue, C, {ev3_3});
 
         // print out first bit of C solution
-        sycl::event ev4 = main_queue.submit([&](sycl::handler &cgh) {
-            cgh.depends_on({ev3_3});
+        sycl::event ev_print = main_queue.submit([&](sycl::handler &cgh) {
+            cgh.depends_on({ev_sort});
             auto kernel = [=]() {
                 std::cout << "C matrix structure [first two rows]:" << std::endl;
                 intType c_ind = c_index == oneapi::mkl::index_base::zero ? 0 : 1;
@@ -364,20 +324,24 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
             cgh.host_task(kernel);
         });
 
-        main_queue.wait();
-
         // clean up
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
+        auto ev_relA = oneapi::mkl::sparse::release_matrix_handle(main_queue, &A, {ev3_3});
+        auto ev_relB = oneapi::mkl::sparse::release_matrix_handle(main_queue, &B, {ev3_3});
+        auto ev_relC = oneapi::mkl::sparse::release_matrix_handle(main_queue, &C, {ev_print});
+
+        ev_relA.wait();
+        ev_relB.wait();
+        ev_relC.wait();
     }
-    catch (cl::sycl::exception const &e) {
+    catch (sycl::exception const &e) {
         std::cout << "\t\tCaught synchronous SYCL exception:\n" << e.what() << std::endl;
+
+        main_queue.wait();
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &A).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &B).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &C).wait();
 
         cleanup_arrays<fpType, intType>(fp_ptr_vec, int_ptr_vec, i64_ptr_vec, void_ptr_vec, cxt);
 
@@ -385,11 +349,13 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     }
     catch (std::exception const &e) {
         std::cout << "\t\tCaught std exception:\n" << e.what() << std::endl;
+
+        main_queue.wait();
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
-    
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &A).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &B).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &C).wait();
+
         cleanup_arrays<fpType, intType>(fp_ptr_vec, int_ptr_vec, i64_ptr_vec, void_ptr_vec, cxt);
 
         return 1;
@@ -443,10 +409,9 @@ void print_example_banner()
 // Main entry point for example.
 //
 // Dispatches to appropriate device types as set at build time with flag:
-// -DSYCL_DEVICES_host -- only runs host implementation
 // -DSYCL_DEVICES_cpu -- only runs SYCL CPU implementation
 // -DSYCL_DEVICES_gpu -- only runs SYCL GPU implementation
-// -DSYCL_DEVICES_all (default) -- runs on all: host, cpu and gpu devices
+// -DSYCL_DEVICES_all (default) -- runs on all: cpu and gpu devices
 //
 //  For each device selected and each supported data type, MatrixMultiplyExample
 //  runs is with all supported data types
@@ -463,12 +428,7 @@ int main(int argc, char **argv)
     int status = 0;
     for (auto it = list_of_devices.begin(); it != list_of_devices.end(); ++it) {
 
-        if (*it == my_sycl_device_types::host_device) {
-            std:: cout << "Host device is not currently supported by sparse::matmat" << std::endl;
-            break;
-        }
-
-        cl::sycl::device my_dev;
+        sycl::device my_dev;
         bool my_dev_is_found = false;
         get_sycl_device(my_dev, my_dev_is_found, *it);
 
@@ -480,7 +440,7 @@ int main(int argc, char **argv)
             if (status != 0)
                 return status;
 
-            if (my_dev.get_info<cl::sycl::info::device::double_fp_config>().size() != 0) {
+            if (my_dev.get_info<sycl::info::device::double_fp_config>().size() != 0) {
                 std::cout << "\tRunning with double precision real data type:" << std::endl;
                 status = run_sparse_matrix_sparse_matrix_multiply_example<double, std::int32_t>(
                         my_dev);
@@ -501,5 +461,6 @@ int main(int argc, char **argv)
         }
     }
 
+    mkl_free_buffers();
     return status;
 }
