@@ -17,7 +17,7 @@
 *  Content:
 *       This example demonstrates use of oneAPI Math Kernel Library (oneMKL)
 *       DPCPP USM API oneapi::mkl::sparse::matmat to perform general
-*       sparse matrix-sparse matrix multiplication on a SYCL device (Host, CPU, GPU).
+*       sparse matrix-sparse matrix multiplication on a SYCL device (CPU, GPU).
 *
 *           C = op(A) * op(B)
 *
@@ -27,7 +27,8 @@
 *
 *       It uses the full control API usage model where the user handles both memory
 *       allocation for the final C Matrix, and allocation of any temporary
-*       workspaces along the way.
+*       workspaces along the way. All matrices use USM device memory which
+*       requires a copy of the data to host at the end for printing results.
 *
 *       The supported floating point data types for matmat matrix data are:
 *           float
@@ -65,7 +66,7 @@
 // square of A.
 //
 template <typename fpType, typename intType>
-int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev)
+int run_sparse_matrix_sparse_matrix_multiply_example(const sycl::device &dev)
 {
     // Initialize data for Sparse Matrix - Sparse Matrix Multiply
     oneapi::mkl::transpose opA = oneapi::mkl::transpose::trans;
@@ -80,12 +81,12 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     oneapi::mkl::index_base c_index = oneapi::mkl::index_base::one;
 
     // Catch asynchronous exceptions
-    auto exception_handler = [](cl::sycl::exception_list exceptions) {
+    auto exception_handler = [](sycl::exception_list exceptions) {
         for (std::exception_ptr const &e : exceptions) {
             try {
                 std::rethrow_exception(e);
             }
-            catch (cl::sycl::exception const &e) {
+            catch (sycl::exception const &e) {
                 std::cout << "Caught asynchronous SYCL "
                              "exception during sparse::gemv:\n"
                           << e.what() << std::endl;
@@ -94,8 +95,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     };
 
     // create execution main_queue
-    cl::sycl::queue main_queue(dev, exception_handler);
-    cl::sycl::context cxt = main_queue.get_context();
+    sycl::queue main_queue(dev, exception_handler);
+    sycl::context cxt = main_queue.get_context();
 
     //
     // set up dimensions of matrix products
@@ -123,91 +124,80 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     //
     // setup A data locally in CSR format
     //
-    std::vector<intType, mkl_allocator<intType, 64>> ia;
-    std::vector<intType, mkl_allocator<intType, 64>> ja;
-    std::vector<fpType, mkl_allocator<fpType, 64>> a;
+    intType *a_rowptr_host = sycl::malloc_host<intType>(a_nrows + 1, main_queue);
+    intType *a_colind_host = sycl::malloc_host<intType>(27 * a_nrows, main_queue);
+    fpType *a_values_host  = sycl::malloc_host<fpType>(27 * a_nrows, main_queue);
+    if (!a_rowptr_host || !a_colind_host || !a_values_host)
+        throw std::runtime_error("Failed to allocate USM memory");
 
-    ia.resize(a_nrows + 1);
-    ja.resize(27 * a_nrows);
-    a.resize(27 * a_nrows);
+    int_ptr_vec.push_back(a_rowptr_host);
+    int_ptr_vec.push_back(a_colind_host);
+    fp_ptr_vec.push_back(a_values_host);
 
     intType a_ind = a_index == oneapi::mkl::index_base::zero ? 0 : 1;
-    generate_sparse_matrix<fpType, intType>(size, ia, ja, a, a_ind);
-    a_nnz = ia[a_nrows] - a_ind;
+    generate_sparse_matrix<fpType, intType>(size, a_rowptr_host, a_colind_host, a_values_host, a_ind);
+    a_nnz = a_rowptr_host[a_nrows] - a_ind;
 
-    intType *a_rowptr, *a_colind;
-    fpType *a_val;
+    intType *a_rowptr = sycl::malloc_device<intType>(a_nrows + 1, main_queue);
+    intType *a_colind = sycl::malloc_device<intType>(a_nnz, main_queue);
+    fpType *a_values  = sycl::malloc_device<fpType>(a_nnz, main_queue);
 
-    a_rowptr = (intType *)malloc_shared((a_nrows + 1) * sizeof(intType), dev, cxt);
-    a_colind = (intType *)malloc_shared((a_nnz) * sizeof(intType), dev, cxt);
-    a_val    = (fpType *)malloc_shared((a_nnz) * sizeof(fpType), dev, cxt);
-
-    if (!a_rowptr || !a_colind || !a_val)
+    if (!a_rowptr || !a_colind || !a_values)
         throw std::runtime_error("Failed to allocate USM memory");
 
     int_ptr_vec.push_back(a_rowptr);
     int_ptr_vec.push_back(a_colind);
-    fp_ptr_vec.push_back(a_val);
+    fp_ptr_vec.push_back(a_values);
 
-    // copy data to USM arrays
-    for (intType i = 0; i < a_nrows + 1; ++i) {
-        a_rowptr[i] = ia[i];
-    }
-    for (intType i = 0; i < a_nnz; ++i) {
-        a_colind[i] = ja[i];
-        a_val[i]    = a[i];
-    }
+    // copy A matrix USM data from host to device
+    auto ev_cpy_ia = main_queue.copy<intType>(a_rowptr_host, a_rowptr, a_nrows + 1);
+    auto ev_cpy_ja = main_queue.copy<intType>(a_colind_host, a_colind, a_nnz);
+    auto ev_cpy_a  = main_queue.copy<fpType>(a_values_host, a_values, a_nnz);
 
     //
     // setup B data locally in CSR format
     //
-    std::vector<intType, mkl_allocator<intType, 64>> ib;
-    std::vector<intType, mkl_allocator<intType, 64>> jb;
-    std::vector<fpType, mkl_allocator<fpType, 64>> b;
+    intType *b_rowptr_host = sycl::malloc_host<intType>(b_nrows + 1, main_queue);
+    intType *b_colind_host = sycl::malloc_host<intType>(27 * b_nrows, main_queue);
+    fpType *b_values_host  = sycl::malloc_host<fpType>(27 * b_nrows, main_queue);
+    if (!b_rowptr_host || !b_colind_host || !b_values_host)
+        throw std::runtime_error("Failed to allocate USM memory");
 
-    ib.resize(b_nrows + 1);
-    jb.resize(27 * b_nrows);
-    b.resize(27 * b_nrows);
+    int_ptr_vec.push_back(b_rowptr_host);
+    int_ptr_vec.push_back(b_colind_host);
+    fp_ptr_vec.push_back(b_values_host);
 
     intType b_ind = b_index == oneapi::mkl::index_base::zero ? 0 : 1;
-    generate_sparse_matrix<fpType, intType>(size, ib, jb, b, b_ind);
-    b_nnz = ib[b_nrows] - b_ind;
+    generate_sparse_matrix<fpType, intType>(size, b_rowptr_host, b_colind_host, b_values_host, b_ind);
+    b_nnz = b_rowptr_host[b_nrows] - b_ind;
 
-    intType *b_rowptr, *b_colind;
-    fpType *b_val;
+    intType *b_rowptr = sycl::malloc_device<intType>(b_nrows + 1, main_queue);
+    intType *b_colind = sycl::malloc_device<intType>(b_nnz, main_queue);
+    fpType *b_values  = sycl::malloc_device<fpType>(b_nnz, main_queue);
 
-    b_rowptr = (intType *)malloc_shared((b_nrows + 1) * sizeof(intType), dev, cxt);
-    b_colind = (intType *)malloc_shared((b_nnz) * sizeof(intType), dev, cxt);
-    b_val    = (fpType *)malloc_shared((b_nnz) * sizeof(fpType), dev, cxt);
-
-    if (!b_rowptr || !b_colind || !b_val)
+    if (!a_rowptr || !b_colind || !b_values)
         throw std::runtime_error("Failed to allocate USM memory");
 
     int_ptr_vec.push_back(b_rowptr);
     int_ptr_vec.push_back(b_colind);
-    fp_ptr_vec.push_back(b_val);
+    fp_ptr_vec.push_back(b_values);
 
-    // copy data to USM arrays
-    for (intType i = 0; i < b_nrows + 1; ++i) {
-        b_rowptr[i] = ib[i];
-    }
-    for (intType i = 0; i < b_nnz; ++i) {
-        b_colind[i] = jb[i];
-        b_val[i]    = b[i];
-    }
+    // copy B matrix USM data from host to device
+    auto ev_cpy_ib = main_queue.copy<intType>(b_rowptr_host, b_rowptr, b_nrows + 1);
+    auto ev_cpy_jb = main_queue.copy<intType>(b_colind_host, b_colind, b_nnz);
+    auto ev_cpy_b  = main_queue.copy<fpType>(b_values_host, b_values, b_nnz);
 
     //
     // setup C data locally in CSR format
     // note: we don't know c_nnz so can only prepare ic at this point
     //
-    intType *c_rowptr = nullptr, *c_colind = nullptr;
-    fpType *c_val = nullptr;
-
-    c_rowptr = (intType *)malloc_shared((c_nrows + 1) * sizeof(intType), dev, cxt);
+    intType c_ind = c_index == oneapi::mkl::index_base::zero ? 0 : 1;
+    intType *c_rowptr = sycl::malloc_device<intType>(c_nrows + 1, main_queue);
+    intType *c_colind = nullptr;
+    fpType *c_values = nullptr;
 
     if (!c_rowptr)
         throw std::runtime_error("Failed to allocate USM memory");
-
     int_ptr_vec.push_back(c_rowptr);
 
     //
@@ -216,6 +206,9 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     std::int64_t *sizeTempBuffer = nullptr, *sizeTempBuffer2 = nullptr, *c_nnz = nullptr;
     void *tempBuffer = nullptr, *tempBuffer2 = nullptr;
 
+    //
+    // Execute Matrix Multiply
+    //
 
     std::cout << "\n\t\tsparse::matmat parameters:\n";
     std::cout << "\t\t\topA = " << opA << std::endl;
@@ -233,15 +226,11 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     std::cout << "\t\t\tB_index = " << b_index << std::endl;
     std::cout << "\t\t\tC_index = " << c_index << std::endl;
 
-    //
-    // Execute Matrix Multiply
-    //
+    oneapi::mkl::sparse::matrix_handle_t A = nullptr;
+    oneapi::mkl::sparse::matrix_handle_t B = nullptr;
+    oneapi::mkl::sparse::matrix_handle_t C = nullptr;
 
-    oneapi::mkl::sparse::matrix_handle_t A;
-    oneapi::mkl::sparse::matrix_handle_t B;
-    oneapi::mkl::sparse::matrix_handle_t C;
-
-    oneapi::mkl::sparse::matmat_descr_t descr = NULL;
+    oneapi::mkl::sparse::matmat_descr_t descr = nullptr;
     oneapi::mkl::sparse::matmat_request req;
 
     try {
@@ -249,14 +238,16 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         oneapi::mkl::sparse::init_matrix_handle(&B);
         oneapi::mkl::sparse::init_matrix_handle(&C);
 
-        oneapi::mkl::sparse::set_csr_data(A, a_nrows, a_ncols, a_index, a_rowptr, a_colind, a_val);
-        oneapi::mkl::sparse::set_csr_data(B, b_nrows, b_ncols, b_index, b_rowptr, b_colind, b_val);
+        auto ev_setA = oneapi::mkl::sparse::set_csr_data(main_queue, A, a_nrows, a_ncols, a_index,
+                a_rowptr, a_colind, a_values, {ev_cpy_ia, ev_cpy_ja, ev_cpy_a});
+        auto ev_setB = oneapi::mkl::sparse::set_csr_data(main_queue, B, b_nrows, b_ncols, b_index,
+                b_rowptr, b_colind, b_values, {ev_cpy_ib, ev_cpy_jb, ev_cpy_b});
 
         //
         // only c_rowptr exists at this point in process so pass in nullptrs
         //
-        oneapi::mkl::sparse::set_csr_data(C, c_nrows, c_ncols, c_index, c_rowptr,
-                                          (intType *)nullptr, (fpType *)nullptr);
+        auto ev_setC = oneapi::mkl::sparse::set_csr_data(main_queue, C, c_nrows, c_ncols, c_index, c_rowptr,
+                                                        (intType *)nullptr, (fpType *)nullptr, {});
 
         //
         // initialize the matmat descriptor
@@ -270,25 +261,25 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 1.1
         //   query for size of work_estimation temp buffer
-        req            = oneapi::mkl::sparse::matmat_request::get_work_estimation_buf_size;
-        sizeTempBuffer = (std::int64_t *)malloc_shared(sizeof(std::int64_t), dev, cxt);
+        req = oneapi::mkl::sparse::matmat_request::get_work_estimation_buf_size;
+        sizeTempBuffer = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!sizeTempBuffer)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(sizeTempBuffer);
 
         auto ev1_1 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer,
-                                                 nullptr, {});
+                                                 nullptr, {ev_setA, ev_setB, ev_setC});
 
         // Step 1.2
         //   allocate temp buffer for work_estimation
         ev1_1.wait();
-        void *tempBuffer = malloc_shared(sizeTempBuffer[0] * sizeof(std::uint8_t), dev, cxt);
+        void *tempBuffer = sycl::malloc_device(sizeTempBuffer[0] * sizeof(std::uint8_t), main_queue);
         if (!tempBuffer)
             throw std::runtime_error("Failed to allocate USM memory");
         void_ptr_vec.push_back(tempBuffer);
 
         // Step 1.3  do work_estimation
-        req        = oneapi::mkl::sparse::matmat_request::work_estimation;
+        req = oneapi::mkl::sparse::matmat_request::work_estimation;
         auto ev1_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer,
                                                  tempBuffer, {ev1_1});
 
@@ -297,8 +288,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         //
 
         // Step 2.1 query size of compute temp buffer
-        req             = oneapi::mkl::sparse::matmat_request::get_compute_buf_size;
-        sizeTempBuffer2 = (std::int64_t *)malloc_shared(sizeof(std::int64_t), dev, cxt);
+        req = oneapi::mkl::sparse::matmat_request::get_compute_buf_size;
+        sizeTempBuffer2 = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!sizeTempBuffer2)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(sizeTempBuffer2);
@@ -307,13 +298,13 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 2.2 allocate temp buffer for compute
         ev2_1.wait();
-        void *tempBuffer2 = malloc_shared(sizeTempBuffer2[0] * sizeof(std::uint8_t), dev, cxt);
+        void *tempBuffer2 = sycl::malloc_device(sizeTempBuffer2[0] * sizeof(std::uint8_t), main_queue);
         if (!tempBuffer2)
             throw std::runtime_error("Failed to allocate USM memory");
         void_ptr_vec.push_back(tempBuffer2);
 
         // Step 2.3 do compute
-        req        = oneapi::mkl::sparse::matmat_request::compute;
+        req = oneapi::mkl::sparse::matmat_request::compute;
         auto ev2_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, sizeTempBuffer2,
                                                  tempBuffer2, {ev2_1});
 
@@ -322,8 +313,8 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
         //
 
         // Step 3.1  get nnz
-        req                 = oneapi::mkl::sparse::matmat_request::get_nnz;
-        std::int64_t *c_nnz = (std::int64_t *)(malloc_shared(1 * sizeof(std::int64_t), dev, cxt));
+        req = oneapi::mkl::sparse::matmat_request::get_nnz;
+        std::int64_t *c_nnz = sycl::malloc_host<std::int64_t>(1, main_queue);
         if (!c_nnz)
             throw std::runtime_error("Failed to allocate USM memory");
         i64_ptr_vec.push_back(c_nnz);
@@ -332,30 +323,56 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
 
         // Step 3.2  allocate final c matrix arrays
         ev3_1.wait();
-        c_colind = (intType *)malloc_shared(c_nnz[0] * sizeof(intType), dev, cxt);
-        c_val    = (fpType *)malloc_shared(c_nnz[0] * sizeof(fpType), dev, cxt);
-        if (!c_colind || !c_val) {
-            throw std::bad_alloc();
+        c_colind = sycl::malloc_device<intType>(c_nnz[0], main_queue);
+        c_values = sycl::malloc_device<fpType>(c_nnz[0], main_queue);
+        if (!c_colind || !c_values) {
+            throw std::runtime_error("Failed to allocate USM memory");
         }
         int_ptr_vec.push_back(c_colind);
-        fp_ptr_vec.push_back(c_val);
-        oneapi::mkl::sparse::set_csr_data(C, c_nrows, c_ncols, c_index, c_rowptr, c_colind, c_val);
+        fp_ptr_vec.push_back(c_values);
+        ev_setC = oneapi::mkl::sparse::set_csr_data(main_queue, C, c_nrows, c_ncols, c_index, c_rowptr, c_colind, c_values, {ev3_1});
 
         // Step 3.3  finalize into C matrix
-        req        = oneapi::mkl::sparse::matmat_request::finalize;
+        req = oneapi::mkl::sparse::matmat_request::finalize;
         auto ev3_3 = oneapi::mkl::sparse::matmat(main_queue, A, B, C, req, descr, nullptr, nullptr,
-                                                 {ev3_1});
+                                                 {ev_setC});
 
-        // print out first bit of C solution
-        sycl::event ev4 = main_queue.submit([&](sycl::handler &cgh) {
-            cgh.depends_on({ev3_3});
+        // Sort C matrix output if desired
+        auto ev_sort = oneapi::mkl::sparse::sort_matrix(main_queue, C, {ev3_3});
+
+        // Copy first set of rows of C to host for printing
+        const intType c_nrows_copy = std::min<intType>(2, c_nrows); // only copy over this many rows of C to host
+
+        intType *c_rowptr_host = sycl::malloc_host<intType>(c_nrows_copy+1, main_queue);
+        if (!c_rowptr_host)
+            throw std::runtime_error("Failed to allocate USM memory");
+        int_ptr_vec.push_back(c_rowptr_host);
+
+        auto ev_cpy_ic = main_queue.copy<intType>(c_rowptr, c_rowptr_host, c_nrows_copy+1, {ev_sort}); // copy part of c_rowptr device to host
+
+        ev_cpy_ic.wait(); // make sure copy is done before reading from it
+        const intType c_host_nnz = c_rowptr_host[c_nrows_copy] - c_ind;
+        intType *c_colind_host = sycl::malloc_host<intType>(c_host_nnz, main_queue);
+        fpType *c_values_host  = sycl::malloc_host<fpType>(c_host_nnz, main_queue);
+
+        if (!c_colind_host || !c_values_host)
+            throw std::runtime_error("Failed to allocate USM memory");
+
+        int_ptr_vec.push_back(c_colind_host);
+        fp_ptr_vec.push_back(c_values_host);
+
+        auto ev_cpy_jc = main_queue.copy<intType>(c_colind, c_colind_host, c_host_nnz, {ev_sort, ev_cpy_ic}); // copy part of c_colind device to host
+        auto ev_cpy_c  = main_queue.copy<fpType>(c_values, c_values_host, c_host_nnz, {ev_sort, ev_cpy_ic}); // copy part of c_values device to host
+
+        // print out a portion of C solution
+        sycl::event ev_print = main_queue.submit([&](sycl::handler &cgh) {
+            cgh.depends_on({ev_cpy_ic, ev_cpy_jc, ev_cpy_c});
             auto kernel = [=]() {
-                std::cout << "C matrix [first two rows]:" << std::endl;
-                intType c_ind = c_index == oneapi::mkl::index_base::zero ? 0 : 1;
-                for (intType row = 0; row < std::min(static_cast<intType>(2), c_nrows); ++row) {
-                    for (intType j = c_rowptr[row] - c_ind; j < c_rowptr[row + 1] - c_ind; ++j) {
-                        intType col = c_colind[j];
-                        fpType val  = c_val[j];
+                std::cout << "C matrix [first " << c_nrows_copy << " rows]:" << std::endl;
+                for (intType row = 0; row < c_nrows_copy; ++row) {
+                    for (intType j = c_rowptr_host[row] - c_ind; j < c_rowptr_host[row + 1] - c_ind; ++j) {
+                        intType col = c_colind_host[j];
+                        fpType val  = c_values_host[j];
                         std::cout << "C(" << row + c_ind << ", " << col << ") = " << val
                                   << std::endl;
                     }
@@ -364,20 +381,24 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
             cgh.host_task(kernel);
         });
 
-        main_queue.wait();
-
         // clean up
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
+        auto ev_relA = oneapi::mkl::sparse::release_matrix_handle(main_queue, &A, {ev3_3});
+        auto ev_relB = oneapi::mkl::sparse::release_matrix_handle(main_queue, &B, {ev3_3});
+        auto ev_relC = oneapi::mkl::sparse::release_matrix_handle(main_queue, &C, {ev_print});
+
+        ev_relA.wait();
+        ev_relB.wait();
+        ev_relC.wait();
     }
-    catch (cl::sycl::exception const &e) {
+    catch (sycl::exception const &e) {
         std::cout << "\t\tCaught synchronous SYCL exception:\n" << e.what() << std::endl;
+
+        main_queue.wait();
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &A).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &B).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &C).wait();
 
         cleanup_arrays<fpType, intType>(fp_ptr_vec, int_ptr_vec, i64_ptr_vec, void_ptr_vec, cxt);
 
@@ -385,13 +406,15 @@ int run_sparse_matrix_sparse_matrix_multiply_example(const cl::sycl::device &dev
     }
     catch (std::exception const &e) {
         std::cout << "\t\tCaught std exception:\n" << e.what() << std::endl;
+
+        main_queue.wait();
         oneapi::mkl::sparse::release_matmat_descr(&descr);
-        oneapi::mkl::sparse::release_matrix_handle(&A);
-        oneapi::mkl::sparse::release_matrix_handle(&B);
-        oneapi::mkl::sparse::release_matrix_handle(&C);
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &A).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &B).wait();
+        oneapi::mkl::sparse::release_matrix_handle(main_queue, &C).wait();
 
         cleanup_arrays<fpType, intType>(fp_ptr_vec, int_ptr_vec, i64_ptr_vec, void_ptr_vec, cxt);
-        
+
         return 1;
     }
 
@@ -443,10 +466,9 @@ void print_example_banner()
 // Main entry point for example.
 //
 // Dispatches to appropriate device types as set at build time with flag:
-// -DSYCL_DEVICES_host -- only runs host implementation
 // -DSYCL_DEVICES_cpu -- only runs SYCL CPU implementation
 // -DSYCL_DEVICES_gpu -- only runs SYCL GPU implementation
-// -DSYCL_DEVICES_all (default) -- runs on all: host, cpu and gpu devices
+// -DSYCL_DEVICES_all (default) -- runs on all: cpu and gpu devices
 //
 //  For each device selected and each supported data type, MatrixMultiplyExample
 //  runs is with all supported data types
@@ -463,12 +485,7 @@ int main(int argc, char **argv)
     int status = 0;
     for (auto it = list_of_devices.begin(); it != list_of_devices.end(); ++it) {
 
-        if (*it == my_sycl_device_types::host_device) {
-            std:: cout << "Host device is not currently supported by sparse::matmat" << std::endl;
-            break;
-        }
-
-        cl::sycl::device my_dev;
+        sycl::device my_dev;
         bool my_dev_is_found = false;
         get_sycl_device(my_dev, my_dev_is_found, *it);
 
@@ -480,7 +497,7 @@ int main(int argc, char **argv)
             if (status != 0)
                 return status;
 
-            if (my_dev.get_info<cl::sycl::info::device::double_fp_config>().size() != 0) {
+            if (my_dev.get_info<sycl::info::device::double_fp_config>().size() != 0) {
                 std::cout << "\tRunning with double precision real data type:" << std::endl;
                 status = run_sparse_matrix_sparse_matrix_multiply_example<double, std::int32_t>(
                         my_dev);
@@ -501,5 +518,6 @@ int main(int argc, char **argv)
         }
     }
 
+    mkl_free_buffers();
     return status;
 }
