@@ -23,6 +23,8 @@
 #define __COMMON_FOR_STATS_EXAMPLES_HPP__
 
 // stl includes
+#include <algorithm>
+#include <random>
 #include <vector>
 
 // local includes
@@ -30,14 +32,18 @@
 
 template<typename Vec>
 void random_vector(Vec &v, int n_observations, int n_dims) {
-    std::srand(1);
-
-    using fp = typename Vec::value_type;
+    using ValueType = typename Vec::value_type;
     v.resize(n_observations * n_dims);
 
-    for(int i = 0; i < n_observations * n_dims; i++) {
-        v[i] = rand_scalar<fp>();
-    }
+    std::random_device rd;
+    std::mt19937 prng(rd());
+
+    std::generate(v.begin(), v.end(),
+        [&]() {
+            double u = prng() * 0x1p-32; // u \in [0; 1)
+            return (1000.0 * (2.0 * u - 1.0)); /// u \in [-1000, +1000)
+        }
+    );
 }
 
 #endif // __COMMON_FOR_STATS_EXAMPLES_HPP__
