@@ -52,20 +52,26 @@ int run_sum_example(sycl::queue& queue) {
 
     sycl::event event;
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
         oneapi::mkl::stats::raw_sum(queue, dataset, sum.data(), raw_sum_2.data(), raw_sum_3.data());
         oneapi::mkl::stats::central_sum(queue, dataset, central_sum_2.data(), central_sum_3.data());
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Sum for dimension " << i << ": " << sum[i] << std::endl;
         std::cout << "raw sum 2 for dimension " << i << ": " << raw_sum_2[i] << std::endl;
         std::cout << "raw sum 3 for dimension " << i << ": " << raw_sum_3[i] << std::endl;
@@ -93,21 +99,28 @@ int run_central_sum_example_with_provided_mean(sycl::queue& queue) {
 
     sycl::event event;
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
         event = oneapi::mkl::stats::mean(queue, dataset, mean.data());
-        oneapi::mkl::stats::central_sum(queue, mean.data(), dataset, central_sum_2.data(), central_sum_3.data(),
-                                        central_sum_4.data(), std::vector<sycl::event>{event});
+        oneapi::mkl::stats::central_sum(queue, mean.data(), dataset, central_sum_2.data(),
+                                        central_sum_3.data(), central_sum_4.data(),
+                                        std::vector<sycl::event>{ event });
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Mean for dimension " << i << ": " << mean[i] << std::endl;
         std::cout << "central sum 2 for dimension " << i << ": " << central_sum_2[i] << std::endl;
         std::cout << "central sum 3 for dimension " << i << ": " << central_sum_3[i] << std::endl;
@@ -134,25 +147,35 @@ int run_moment_example(sycl::queue& queue) {
     std::vector<RealType, decltype(allocator)> central_moment_3(n_dims, allocator);
 
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
-        oneapi::mkl::stats::raw_moment(queue, dataset, mean.data(), raw_moment_2.data(), raw_moment_3.data());
-        oneapi::mkl::stats::central_moment(queue, dataset, central_moment_2.data(), central_moment_3.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
+        oneapi::mkl::stats::raw_moment(queue, dataset, mean.data(), raw_moment_2.data(),
+                                       raw_moment_3.data());
+        oneapi::mkl::stats::central_moment(queue, dataset, central_moment_2.data(),
+                                           central_moment_3.data());
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Mean for dimension " << i << ": " << mean[i] << std::endl;
         std::cout << "raw moment 2 for dimension " << i << ": " << raw_moment_2[i] << std::endl;
         std::cout << "raw moment 3 for dimension " << i << ": " << raw_moment_3[i] << std::endl;
-        std::cout << "central moment 2 for dimension " << i << ": " << central_moment_2[i] << std::endl;
-        std::cout << "central moment 3 for dimension " << i << ": " << central_moment_3[i] << std::endl;
+        std::cout << "central moment 2 for dimension " << i << ": " << central_moment_2[i]
+                  << std::endl;
+        std::cout << "central moment 3 for dimension " << i << ": " << central_moment_3[i]
+                  << std::endl;
     }
     return 0;
 }
@@ -174,26 +197,35 @@ int run_central_moment_example_with_provided_mean(sycl::queue& queue) {
     std::vector<RealType, decltype(allocator)> central_moment_4(n_dims, allocator);
 
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
         sycl::event event = oneapi::mkl::stats::mean(queue, dataset, mean.data());
-        oneapi::mkl::stats::central_moment(queue, mean.data(), dataset, central_moment_2.data(), central_moment_3.data(),
-                                          central_moment_4.data(), std::vector<sycl::event>{event});
+        oneapi::mkl::stats::central_moment(queue, mean.data(), dataset, central_moment_2.data(),
+                                           central_moment_3.data(), central_moment_4.data(),
+                                           std::vector<sycl::event>{ event });
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
-
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Mean for dimension " << i << ": " << mean[i] << std::endl;
-        std::cout << "central moment 2 for dimension " << i << ": " << central_moment_2[i] << std::endl;
-        std::cout << "central moment 3 for dimension " << i << ": " << central_moment_3[i] << std::endl;
-        std::cout << "central moment 4 for dimension " << i << ": " << central_moment_4[i] << std::endl;
+        std::cout << "central moment 2 for dimension " << i << ": " << central_moment_2[i]
+                  << std::endl;
+        std::cout << "central moment 3 for dimension " << i << ": " << central_moment_3[i]
+                  << std::endl;
+        std::cout << "central moment 4 for dimension " << i << ": " << central_moment_4[i]
+                  << std::endl;
     }
     return 0;
 }
@@ -214,21 +246,27 @@ int run_skewness_kurtosis_variation_example(sycl::queue& queue) {
     std::vector<RealType, decltype(allocator)> variation(n_dims, allocator);
 
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
         oneapi::mkl::stats::skewness(queue, dataset, skewness.data());
         oneapi::mkl::stats::kurtosis(queue, dataset, kurtosis.data());
         oneapi::mkl::stats::variation(queue, dataset, variation.data());
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Skewness for dimension " << i << ": " << skewness[i] << std::endl;
         std::cout << "Kurtosis for dimension " << i << ": " << kurtosis[i] << std::endl;
         std::cout << "Variation for dimension " << i << ": " << variation[i] << std::endl;
@@ -254,22 +292,31 @@ int run_skewness_kurtosis_variation_example_with_provided_mean(sycl::queue& queu
 
     sycl::event event;
     try {
-        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(n_dims, n_observations, x.data());
+        auto dataset = oneapi::mkl::stats::make_dataset<oneapi::mkl::stats::layout::row_major>(
+            n_dims, n_observations, x.data());
         event = oneapi::mkl::stats::mean(queue, dataset, mean.data());
-        oneapi::mkl::stats::skewness(queue, mean.data(), dataset, skewness.data(), std::vector<sycl::event>{event});
-        oneapi::mkl::stats::kurtosis(queue, mean.data(), dataset, kurtosis.data(), std::vector<sycl::event>{event});
-        oneapi::mkl::stats::variation(queue, mean.data(), dataset, variation.data(), std::vector<sycl::event>{event});
+        oneapi::mkl::stats::skewness(queue, mean.data(), dataset, skewness.data(),
+                                     std::vector<sycl::event>{ event });
+        oneapi::mkl::stats::kurtosis(queue, mean.data(), dataset, kurtosis.data(),
+                                     std::vector<sycl::event>{ event });
+        oneapi::mkl::stats::variation(queue, mean.data(), dataset, variation.data(),
+                                      std::vector<sycl::event>{ event });
         queue.wait_and_throw();
     }
-    catch(sycl::exception const& e) {
+    catch (sycl::exception const& e) {
         std::cout << "\t\tSYCL exception during calculation\n"
-                  << e.what() << std::endl << "OpenCL status: " << get_error_code(e) << std::endl;
+                  << e.what() << std::endl
+                  << "Error code: " << get_error_code(e) << std::endl;
+        return 1;
+    }
+    catch (oneapi::mkl::exception const& e) {
+        std::cout << "\toneMKL exception during calculation\n" << e.what() << std::endl;
         return 1;
     }
 
     std::cout << "Number of dimensions: " << n_dims << std::endl;
     std::cout << "Number of observations: " << n_observations << std::endl;
-    for(int i = 0; i < n_dims; i++) {
+    for (int i = 0; i < n_dims; i++) {
         std::cout << "Skewness for dimension " << i << ": " << skewness[i] << std::endl;
         std::cout << "Kurtosis for dimension " << i << ": " << kurtosis[i] << std::endl;
         std::cout << "Variation for dimension " << i << ": " << variation[i] << std::endl;
@@ -282,41 +329,43 @@ int run_skewness_kurtosis_variation_example_with_provided_mean(sycl::queue& queu
 //
 void print_example_banner() {
     std::cout << "" << std::endl;
-    std::cout << "########################################################################" << std::endl;
+    std::cout << "########################################################################"
+              << std::endl;
     std::cout << "# Basic statistics estimates example: " << std::endl;
     std::cout << "# " << std::endl;
     std::cout << "# Using apis:" << std::endl;
     std::cout << "# oneapi::mkl::stats::raw_sum oneapi::mkl::stats::central_sum" << std::endl;
     std::cout << "# oneapi::mkl::stats::raw_moment oneapi::mkl::stats::central_moment" << std::endl;
-    std::cout << "# oneapi::mkl::stats::mean oneapi::mkl::stats::skewness oneapi::mkl::stats::kurtosis oneapi::mkl::stats::variance" << std::endl;
+    std::cout
+        << "# oneapi::mkl::stats::mean oneapi::mkl::stats::skewness oneapi::mkl::stats::kurtosis oneapi::mkl::stats::variance"
+        << std::endl;
     std::cout << "# " << std::endl;
     std::cout << "# Supported floating point type precisions:" << std::endl;
     std::cout << "#   float double" << std::endl;
     std::cout << "# " << std::endl;
-    std::cout << "########################################################################" << std::endl;
+    std::cout << "########################################################################"
+              << std::endl;
     std::cout << std::endl;
 }
 
-
-
 //
 // Main entry point for example.
 //
 // Dispatches to appropriate device types as set at build time with flag:
-// -DSYCL_DEVICES_host -- only runs host implementation
 // -DSYCL_DEVICES_cpu -- only runs SYCL CPU implementation
 // -DSYCL_DEVICES_gpu -- only runs SYCL GPU implementation
-// -DSYCL_DEVICES_all (default) -- runs on all: host, cpu and gpu devices
+// -DSYCL_DEVICES_all (default) -- runs on all: cpu and gpu devices
 //
 
-int main (int argc, char ** argv) {
-    auto exception_handler = [] (sycl::exception_list exceptions) {
+int main(int argc, char** argv) {
+    auto exception_handler = [](sycl::exception_list exceptions) {
         for (std::exception_ptr const& e : exceptions) {
             try {
                 std::rethrow_exception(e);
-            } catch(sycl::exception const& e) {
+            }
+            catch (sycl::exception const& e) {
                 std::cout << "Caught asynchronous SYCL exception during calculation:\n"
-                << e.what() << std::endl;
+                          << e.what() << std::endl;
             }
         }
     };
@@ -327,48 +376,50 @@ int main (int argc, char ** argv) {
     set_list_of_devices(list_of_devices);
 
     for (auto it = list_of_devices.begin(); it != list_of_devices.end(); ++it) {
-
         sycl::device my_dev;
         bool my_dev_is_found = false;
         get_sycl_device(my_dev, my_dev_is_found, *it);
 
-            if(my_dev_is_found) {
-                sycl::queue queue(my_dev, exception_handler);
-                std::cout << "Running tests on " << sycl_device_names[*it] << ".\n";
-                std::cout << "\tRunning with single precision real data type:" << std::endl;
-                if(run_sum_example<float>(queue) ||
-                    run_central_sum_example_with_provided_mean<float>(queue) ||
-                    run_moment_example<float>(queue) ||
-                    run_central_moment_example_with_provided_mean<float>(queue) ||
-                    run_skewness_kurtosis_variation_example<float>(queue) ||
-                    run_skewness_kurtosis_variation_example_with_provided_mean<float>(queue)) {
+        if (my_dev_is_found) {
+            sycl::queue queue(my_dev, exception_handler);
+            std::cout << "Running tests on " << sycl_device_names[*it] << ".\n";
+            std::cout << "\tRunning with single precision real data type:" << std::endl;
+            if (run_sum_example<float>(queue) ||
+                run_central_sum_example_with_provided_mean<float>(queue) ||
+                run_moment_example<float>(queue) ||
+                run_central_moment_example_with_provided_mean<float>(queue) ||
+                run_skewness_kurtosis_variation_example<float>(queue) ||
+                run_skewness_kurtosis_variation_example_with_provided_mean<float>(queue)) {
+                std::cout << "FAILED" << std::endl;
+                return 1;
+            }
+            if (isDoubleSupported(my_dev)) {
+                std::cout << "\tRunning with double precision real data type:" << std::endl;
+                if (run_sum_example<double>(queue) ||
+                    run_central_sum_example_with_provided_mean<double>(queue) ||
+                    run_moment_example<double>(queue) ||
+                    run_central_moment_example_with_provided_mean<double>(queue) ||
+                    run_skewness_kurtosis_variation_example<double>(queue) ||
+                    run_skewness_kurtosis_variation_example_with_provided_mean<double>(queue)) {
                     std::cout << "FAILED" << std::endl;
                     return 1;
                 }
-                if(isDoubleSupported(my_dev)) {
-                    std::cout << "\tRunning with double precision real data type:" << std::endl;
-                    if(run_sum_example<double>(queue) ||
-                       run_central_sum_example_with_provided_mean<double>(queue) ||
-                       run_moment_example<double>(queue) ||
-                       run_central_moment_example_with_provided_mean<double>(queue) ||
-                       run_skewness_kurtosis_variation_example<double>(queue) ||
-                       run_skewness_kurtosis_variation_example_with_provided_mean<double>(queue)) {
-                        std::cout << "FAILED" << std::endl;
-                        return 1;
-                    }
-                }
-                else {
-                    std::cout << "Double precision is not supported for this device" << std::endl;
-                }
-            } else {
-#ifdef FAIL_ON_MISSING_DEVICES
-                std::cout << "No " << sycl_device_names[*it] << " devices found; Fail on missing devices is enabled.\n";
-                std::cout << "FAILED" << std::endl;
-                return 1;
-#else
-                std::cout << "No " << sycl_device_names[*it] << " devices found; skipping " << sycl_device_names[*it] << " tests.\n";
-#endif
             }
+            else {
+                std::cout << "Double precision is not supported for this device" << std::endl;
+            }
+        }
+        else {
+#ifdef FAIL_ON_MISSING_DEVICES
+            std::cout << "No " << sycl_device_names[*it]
+                      << " devices found; Fail on missing devices is enabled.\n";
+            std::cout << "FAILED" << std::endl;
+            return 1;
+#else
+            std::cout << "No " << sycl_device_names[*it] << " devices found; skipping "
+                      << sycl_device_names[*it] << " tests.\n";
+#endif
+        }
     }
     std::cout << "PASSED" << std::endl;
     return 0;
